##
# File:    MessagingDataImport.py
# Date:    21-Sep-2010
#
# Update:
#
# 2013-09-09    RPS    Ported here from EditorDataImport
# 2013-09-20    RPS    Now method for accommodating model file in PDB format as well as cif/pdbx format.
# 2013-09-23    RPS    Updates to extend handling of milestone file references. Added "deposit" as valid file source.
# 2013-09-27    RPS    Streamlined interface.
# 2013-10-30    RPS    Support for "Notes" UI
# 2014-07-15    RPS    Updates to support upload of more than one auxilary file in annot messaging UI
# 2014-09-17    RPS    Addressed bug regarding failure to provide path to file for contentTypes where "createAsNeeded" in play
# 2016-09-14    ZF     Added support for group deposition message files
# 2018-04-23    EP     Add logging
##

"""
Class to encapsulate data import for files requested by Messaging Module from the workflow directory hierarchy.

"""
__docformat__ = "restructuredtext en"
__author__ = "Raul Sala"
__email__ = "rsala@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys
import os
import os.path


from wwpdb.io.locator.DataReference import DataFileReference

import logging

logger = logging.getLogger(__name__)


class MessagingDataImport(object):
    """Controlling class for data import operations

    Supported file sources:
    + archive         -  WF archive storage
    + wf-instance     -  WF instance storage
    + deposit         -  WF deposit storage

    """

    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        self.__verbose = verbose
        self.__reqObj = reqObj
        # self.__lfh = log
        #
        self.__sessionObj = None
        #
        logger.debug("Starting")
        #
        self.__setup()
        #

    def __setup(self):

        try:
            self.__sessionObj = self.__reqObj.getSessionObj()
            # self.__sessionPath = self.__sessionObj.getPath()
            self.__identifier = str(self.__reqObj.getValue("identifier")).upper()
            self.__instance = str(self.__reqObj.getValue("instance")).upper()
            self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
            self.__fileSource = "archive"  # fixing value to "archive" for now
            # """
            # self.__fileSource  = str(self.__reqObj.getValue("filesource")).lower()
            # if self.__fileSource not in ['archive','wf-archive','wf-instance','wf_archive','wf_instance']:
            #     self.__fileSource = 'archive'
            # """
            #
            # Added by ZF
            #
            self.__groupId = str(self.__reqObj.getValue("groupid")).upper()
            if (not self.__groupId) and len(self.__instance) == 9 and self.__instance[0:2] == "G_":
                self.__groupId = self.__instance
                self.__fileSource = "autogroup"
            #
            if self.__verbose:
                logger.debug("file source %s", self.__fileSource)
                logger.debug("identifier  %s", self.__identifier)
                logger.debug("instance    %s", self.__instance)
        except:  # noqa: E722  pylint: disable=bare-except
            logger.exception("sessionId %s failed", self.__sessionObj.getId())

    def getFilePath(self, contentType, format):  # pylint: disable=redefined-builtin
        createAsNeeded = True if (contentType in ["messages-from-depositor", "messages-to-depositor", "notes-from-annotator"]) else False
        return self.__getWfFilePath(contentType=contentType, fmt=format, fileSource=self.__fileSource, version="latest", createAsNeeded=createAsNeeded)

    # ########################--Milestone File Handling--########################################################
    def getMileStoneFilePaths(self, contentType, format, version="latest", partitionNum=None):  # pylint: disable=redefined-builtin
        pathDict = {}
        pathDict["dpstPth"] = self.__getWfFilePath(contentType=contentType, fmt=format, fileSource="deposit", version=version)
        pathDict["annotPth"] = self.__getWfFilePath(
            contentType=contentType, fmt=format, fileSource=self.__fileSource, version=version, createAsNeeded=False, partitionNum=partitionNum
        )
        return pathDict

    ###########################################################################################################

    def __getWfFilePath(self, contentType, fmt="pdbx", fileSource="archive", version="latest", createAsNeeded=False, partitionNum=None):
        try:
            fPath = self.__getWfFilePathRef(contentType=contentType, fmt=fmt, fileSource=fileSource, version=version, partitionNum=partitionNum)
            logger.debug("checking %s path %s", contentType, fPath)
            if fPath:
                if os.access(fPath, os.R_OK):
                    return fPath
                else:
                    if createAsNeeded:
                        try:
                            f = open(fPath, "w")
                            if os.access(fPath, os.R_OK):
                                f.close()
                                return fPath
                        except IOError:
                            logger.exception("Unable to create '%s' file for id [%s]", contentType, self.__identifier)
                    else:
                        return None
            else:
                return None
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Failed to getWfFilePath")
            return None

    def __getWfFilePathRef(self, contentType, fmt="pdbx", fileSource="archive", version="latest", partitionNum=None):
        """Return the path to the latest version of the contentType"""

        dfRef = DataFileReference(siteId=self.__siteId)
        logger.debug("site id is %s", dfRef.getSitePrefix())

        dfRef.setDepositionDataSetId(self.__identifier)
        if fileSource in ["archive", "wf-archive", "wf_archive"]:
            dfRef.setStorageType("archive")
        elif fileSource == "deposit":
            dfRef.setStorageType("deposit")
        elif fileSource in ["wf-instance", "wf_instance"]:
            dfRef.setWorkflowInstanceId(self.__instance)
            dfRef.setStorageType("wf-instance")
        else:
            logger.error("Bad file source for %s id %s wf id %s", contentType, self.__identifier, self.__instance)
        #
        # Added by ZF
        #
        if self.__groupId and contentType in ["messages-from-depositor", "messages-to-depositor", "notes-from-annotator"]:
            dfRef.setDepositionDataSetId(self.__groupId)
            dfRef.setStorageType("autogroup")
        #
        dfRef.setContentTypeAndFormat(contentType, fmt)
        dfRef.setVersionId(version)
        #
        if partitionNum:
            dfRef.setPartitionNumber(partitionNum)
        #
        fP = None
        if dfRef.isReferenceValid():
            dP = dfRef.getDirPathReference()
            fP = dfRef.getFilePathReference()
            if self.__verbose:
                logger.debug("file directory path: %s", dP)
                logger.debug("file           path: %s", fP)
        else:
            logger.debug("bad reference for %s id %s wf id %s", contentType, self.__identifier, self.__instance)

        #
        return fP


if __name__ == "__main__":
    di = MessagingDataImport()



##
# File:    Message.py
# Author:  R. Sala
# Date:    03-Dec-2015
# Version: 0.001 Initial version
#
# Updates:
#    03-Dec-2015    RPS    Created.
##
"""
A module to define models for different kinds of Message entities

"""
__docformat__ = "restructuredtext en"
__author__ = "Raul Sala"
__email__ = "rsala@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.001"

import sys
import logging

from wwpdb.apps.msgmodule.io.MessagingDataImport import MessagingDataImport

logger = logging.getLogger(__name__)


class Message(object):
    msgsContentType = "msgs"
    defaultMsgsToDpstrFilePath = "/net/wwpdb_da/da_top/wwpdb_da_test/source/python/pdbx_v2/message/messages-to-depositor.cif"
    defaultMsgsFrmDpstrFilePath = "/net/wwpdb_da/da_top/wwpdb_da_test/source/python/pdbx_v2/message/messages-from-depositor.cif"
    defaultNotesFilePath = "/net/wwpdb_da/da_top/wwpdb_da_test/source/python/pdbx_v2/message/notes-from-annotator.cif"

    @staticmethod
    def encodeUtf8ToCif(p_content):
        """Encoding unicode/utf-8 content into cif friendly ascii
        Have to replace any ';' that begin a newline with a ' ;' in order to preserve ; matching required for multiline items
        """
        if sys.version_info[0] < 3:
            return p_content.encode("ascii", "xmlcharrefreplace").replace("\n;", "\n ;").replace("\\\\xa0", " ")
        else:
            return p_content.encode("ascii", "xmlcharrefreplace").decode("ascii").replace("\n;", "\n ;").replace("\\\\xa0", " ")

    @staticmethod
    def fromReqObj(p_reqObj, verbose=True, log=sys.stderr):
        """Factory method that takes input requestObject and depending on the given request parameters
        creates a Message object from Message super class or one of its subclasses (Note, ReminderMessage)

        :Params:
            :param `p_reqObj`: request object
            :param `verbose`: True or False
            :param `log`: handle to file for logging output

        :Returns:
            Message object

        """
        msgDict = {}

        msgDict["deposition_data_set_id"] = p_reqObj.getValue("identifier")
        msgDict["sender"] = p_reqObj.getValue("sender")
        msgDict["message_subject"] = Message.encodeUtf8ToCif(p_reqObj.getRawValue("subject"))
        msgDict["message_text"] = Message.encodeUtf8ToCif(p_reqObj.getRawValue("message"))

        #
        msgType = p_reqObj.getValue("message_type")
        msgDict["message_type"] = msgType if msgType is not None and len(msgType) > 0 else "text"  # "archive" | "archive_manual" | "archive_flag" | "forward" | "forward_manual"

        # below two fields not currently used but may come into play in future
        msgDict["context_type"] = p_reqObj.getValue("context_type")
        msgDict["context_value"] = p_reqObj.getValue("context_value")

        sendStatus = p_reqObj.getValue("send_status")
        msgDict["send_status"] = (
            sendStatus if sendStatus else "Y"
        )  # set to "Y" or "N" for "live" vs. "draft", in all other contexts such as archive,forward,autoMsg --> this is "Y"

        # message_state is artificially introduced into p_reqObj by MessagingWebApp to distinguish "livemsg" from "draft"
        # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.
        msgDict["message_state"] = p_reqObj.getValue("message_state")

        msgId = p_reqObj.getValue("msg_id")
        if msgDict["message_state"] == "draft" and msgId and len(msgId) > 1:
            msgDict["message_id"] = msgId

        parentMsgId = p_reqObj.getValue("parent_msg_id")  # if this msg was in response to another msg
        if msgDict["message_state"] == "livemsg" and parentMsgId and len(parentMsgId) > 1:
            msgDict["parent_message_id"] = parentMsgId

        contentType = p_reqObj.getValue("content_type")  # "notes" vs. "msgs"

        fileRefLst = p_reqObj.getValueList("msg_file_references")  # msg_file_references includes file references selected via checkboxes in UI PLUS those references that represent
        # auxiliary file uploads and which are added via processing in  MessingWebApp

        msgCategory = p_reqObj.getValue("msg_category") if p_reqObj.getValue("msg_category") is not None else ""  # in support of "reminder" category of emails

        if contentType == "notes":
            return Note(msgDict, fileRefLst, verbose, log)
        else:
            if msgCategory == "reminder":
                rtrnMsg = ReminderMessage(msgDict, fileRefLst, verbose, log)
            else:
                rtrnMsg = Message(msgDict, fileRefLst, verbose, log)

            return rtrnMsg

    def __init__(self, p_msgDict, p_fileRefLst=None, p_verbose=True, p_log=sys.stderr):
        """__init__

        :Params:
            :param `p_msgDict`: dictionary of items representing the given message entity
            :param `p_fileRefLst`: list of any file references ("attachments") associated with the message entity
            :param `p_verbose`: True or False
            :param `p_log`: handle to file for logging output

        :Returns:
            na

        """
        if p_fileRefLst is None:
            p_fileRefLst = []
        self._verbose = p_verbose
        self._lfh = p_log
        #
        self._msgDict = p_msgDict.copy()
        #
        self._bIsAutoMsg = False
        self._bIsReminderMsg = False
        # Should the MessageClass force sending of email (note)
        self._bIsNoteEmail = False
        #
        self.fileReferences = p_fileRefLst

    def getMsgDict(self):
        """Returns copy of dictionary that represents the message being processed

        :Params:
            :param na

        :Returns:
            copy of object's message dictionary

        """
        return self._msgDict.copy()

    @property
    def isLive(self):
        """Is the message actually being sent (as opposed to being just a draft at this point in time)

        :Params:
            :param na

        :Returns:
            boolean indicating whether message is live, i.e. actually being sent to depositor

        """
        return self._msgDict["message_state"] == "livemsg"
        # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.

    @property
    def isDraft(self):
        """Is the message just a draft at this point in time

        :Params:
            :param na

        :Returns:
            boolean indicating whether message is a "draft"

        """
        return self._msgDict["message_state"] == "draft"
        # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.

    @property
    def isAutoMsg(self):
        """Is the message one that is being automatically sent by another server-side python module (e.g. Release module)
        (i.e. as opposed to being invoked via URL request)

        :Params:
            :param na

        :Returns:
            boolean indicating whether message is a an "auto" msg

        """
        return self._bIsAutoMsg is True

    @property
    def isReminderMsg(self):
        """Is the message one that is being generated via URL request to have a reminder message propagated to depositor

        :Params:
            :param na

        :Returns:
            boolean indicating whether message is a an "reminder" msg

        """
        return self._bIsReminderMsg is True

    @property
    def contentType(self):
        """Property returning the contentType of the message
        (i.e. "msgs" vs "notes" type, which determines which datafile the message entity is persisted to)

        :Params:
            :param na

        :Returns:
            String indicating "msgs" | "notes"

        """
        return Message.msgsContentType
        # return "msgs"

    @property
    def messageSubject(self):
        """Property returning the subject line of the message

        :Params:
            :param na

        :Returns:
            String representing subject line of the message

        """
        return self._msgDict["message_subject"]

    @property
    def messageText(self):
        """Property returning the textual body of the message

        :Params:
            :param na

        :Returns:
            String representing the message's content body

        """
        return self._msgDict["message_text"]

    @property
    def messageId(self):
        """Property returning the unique ID of the message

        :Params:
            :param na

        :Returns:
            String representing the message's unique ID

        """
        return self._msgDict["message_id"]

    @messageId.setter
    def messageId(self, p_msgId):
        """Property setter allowing assignment of unique ID for the message

        :Params:
            :param p_msgId - message ID to be assigned to this message

        :Returns:
            na

        """
        self._msgDict["message_id"] = p_msgId

    @property
    def depositionId(self):
        """Property returning the ID of deposition to which the message relates

        :Params:
            :param na

        :Returns:
            String representing the ID of deposition to which the message relates

        """
        return self._msgDict["deposition_data_set_id"]

    @property
    def sendStatus(self):
        """Property returning sendStatus of the message
        i.e. "Y" | "N"  indicating whether or not the message has actually been sent or not
        will be "N" in cases where a draft is being saved or updated (and not actually sent)

        :Params:
            :param na

        :Returns:
            String representing the "Y" or "N" send status

        """
        return self._msgDict["send_status"]

    @property
    def messageType(self):
        """Property returning "type" of the message

        Can be one of the following:
            "text"               - default, conventional msg that is captured in messages data file (seen by both annotator and depositor), OR
                                   conventional note authored by annotator and captured in notes data file (seen by annotator only)
            "archive{_auto}"     - msg that is archived to notes data file via URL request (called by email server processing), does not trigger flag for annotator attention
            "archive_manual"     - msg that is archived to notes data file via manual user interaction with UI, triggers flag for annotator attention
            "archive_auto_noorig"- msg that is archived to notes via automatic server process (not by email handler), but does not contain _pdbx_deposition_message_origcomm_reference
            "archive_flag"       - msg that is archived to notes data file via URL request (called by email server processing), but needs to be flagged for annotator attention
            "forward"            - msg that is forwarded into system via URL request (called by email server processing), captured in messages data file,
                                   seen by both annotator and depositor
            "forward_manual"     - msg that is forwarded via manual user interaction with UI, captured in messages data file, seen by both annotator and depositor

        :Params:
            :param na

        :Returns:
            String representing the type of message

        """
        return self._msgDict["message_type"]

    @property
    def isBeingSent(self):
        """Property returning boolean indicating whether the message is being sent

        :Params:
            :param na

        :Returns:
            Boolean indicating whether message is being sent (as opposed to remaining in draft form)

        """
        return self._msgDict["send_status"] == "Y"

    def getFileReferences(self):
        """method that returns list of any file references ("attachments") associated with the message entity

        :Params:
            :param na

        :Returns:
            list of any file references ("attachments") associated with the message entity

        """
        return self.fileReferences

    def setFileReferences(self, p_fileRefsLst):
        """method that sets list of any file references ("attachments") associated with the message entity

        :Params:
            :param p_fileRefsLst

        :Returns:
            na

        """
        self.fileReferences = p_fileRefsLst

    def getOutputFileTarget(self, p_reqObj):
        """for given deposition, determine path to cif messaging data file to be updated

        :Params:
            :param `p_reqObj` : request Object

        :Returns:
            returnFilePath : absolute path to cif data file containing messages or notes for the given deposition

        """
        returnFilePath = None

        msgDI = MessagingDataImport(p_reqObj, verbose=self._verbose, log=self._lfh)

        returnFilePath = msgDI.getFilePath(contentType="messages-to-depositor", format="pdbx") if self._isWorkflow(p_reqObj) else Message.defaultMsgsToDpstrFilePath
        logger.info("messages-to-depositor path is: %s", returnFilePath)

        return returnFilePath

    def _isWorkflow(self, p_reqObj):
        """Determine if currently operating in Workflow Managed environment

        :Returns:
            boolean indicating whether or not currently operating in Workflow Managed environment
        """
        #
        fileSource = str(p_reqObj.getValue("filesource")).lower()
        #
        if fileSource and fileSource in ["archive", "wf-archive", "wf_archive", "wf-instance", "wf_instance"]:
            # if the file source is any of the above then we are in the workflow manager environment
            return True
        else:
            # else we are in the standalone dev environment
            return False

    @property
    def isNoteEmail(self):
        """Is the message is being automatically sent by another server-side python module (e.g. reminder), but
        does not normally trigger an email message, force sending

        :Params:
            :param na

        :Returns:
            boolean indicating whether message should send email

        """
        return self._bIsNoteEmail is True


class AutoMessage(Message):
    """Subclass to represent message that is being automatically sent by another server-side python module (e.g. Release module)
    (i.e. as opposed to being invoked via URL request by a web app client )

    """

    def __init__(self, p_msgDict, p_fileRefLst=None, p_verbose=True, p_log=sys.stderr):
        if p_fileRefLst is None:
            p_fileRefLst = []
        Message.__init__(self, p_msgDict, p_fileRefLst, p_verbose, p_log)
        #
        self._bIsAutoMsg = True
        self._msgDict["message_state"] = "livemsg"  # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.


class ReminderMessage(Message):
    """Subclass to represent release date "reminder" messages that require different handling (e.g. for HPUB entries). These are invoked via URL request."""

    def __init__(self, p_msgDict, p_fileRefLst=None, p_verbose=True, p_log=sys.stderr):
        if p_fileRefLst is None:
            p_fileRefLst = []
        Message.__init__(self, p_msgDict, p_fileRefLst, p_verbose, p_log)
        #
        self._bIsReminderMsg = True
        self._msgDict["message_state"] = "livemsg"  # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.


class Note(Message):
    """Subclass to represent "Note" message content type, which are persisted to a separate notes data file."""

    notesContentType = "notes"

    def getOutputFileTarget(self, p_reqObj):
        """for given deposition, determine path to cif messaging data file to be updated

        :Params:
            :param `reqObj` : request Object

        :Returns:
            returnFilePath : absolute path to cif data file containing messages or notes for the given deposition
        """
        returnFilePath = None

        msgDI = MessagingDataImport(p_reqObj, verbose=self._verbose, log=self._lfh)

        returnFilePath = msgDI.getFilePath(contentType="notes-from-annotator", format="pdbx") if self._isWorkflow(p_reqObj) else Message.defaultNotesFilePath
        logger.info("-- notes-from-annotator path is: %s", returnFilePath)

        return returnFilePath

    @property
    def contentType(self):
        return Note.notesContentType
        # return "notes"


class AutoNote(Note):
    """Subclass to represent message that is being automatically sent by another server-side python module and destined for notes (e.g. Release module)
    (i.e. as opposed to being invoked via URL request by a web app client )
    Support for optional email sending of message as well
    """

    def __init__(self, p_msgDict, p_fileRefLst=None, p_verbose=True, p_log=sys.stderr, p_email=True):

        if p_fileRefLst is None:
            p_fileRefLst = []

        Note.__init__(self, p_msgDict, p_fileRefLst, p_verbose, p_log)
        #
        self._bIsAutoMsg = True
        self._bIsNoteEmail = p_email
        self._msgDict["message_state"] = "livemsg"  # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.
        self._msgDict["message_type"] = "archive_auto_noorig"  # Archive a note, but not original message



##
# File:  ExtractMessage.py
# Date:  28-Jun-2019 E. Peisach
#
# Update:
"""
Support for automatic extracting information from send messages


"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "peisach@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"

import os
import sys
import datetime
import logging
import re
from mmcif.io.PdbxReader import PdbxReader
from wwpdb.io.locator.PathInfo import PathInfo

#

logger = logging.getLogger(__name__)


class ExtractMessage(object):
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        self.__siteId = siteId
        self.__verbose = verbose
        self.__log = log
        self.__pI = PathInfo(siteId=self.__siteId, verbose=self.__verbose, log=self.__log)

    def getLastUnlocked(self, depid):
        """Returns datetime of the last time an unlocked message was sent.  This is parallel to the code in
        MessagingIo.py

        If no unlock message has been sent, will return None
        """

        msgfile = self.__pI.getFilePath(depid, contentType="messages-to-depositor", formatType="pdbx", fileSource="archive", versionId="1")

        if not os.path.exists(msgfile):
            return None

        ret = None

        myContainerList = []
        ifh = open(msgfile, "r")
        pRd = PdbxReader(ifh)
        pRd.read(myContainerList)

        if len(myContainerList) >= 1:
            c0 = myContainerList[0]
            catObj = c0.getObj("pdbx_deposition_message_info")
            if catObj is None:
                logger.debug("Deposition %s no pdbx_deposition_message_info category", depid)
                return None
            else:
                #
                # Get column name index.
                #
                itDict = {}
                itNameList = catObj.getItemNameList()
                for idxIt, itName in enumerate(itNameList):
                    itDict[str(itName).lower()] = idxIt
                    #
                idxOrdinalId = itDict["_pdbx_deposition_message_info.ordinal_id"]
                idxLastCommDate = itDict["_pdbx_deposition_message_info.timestamp"]
                idxMsgSubj = itDict["_pdbx_deposition_message_info.message_subject"]

                maxUnlockOrdId = 0
                for row in catObj.getRowList():
                    try:
                        ordinalId = int(row[idxOrdinalId])
                        msgsubj = row[idxMsgSubj]

                        if msgsubj == "System Unlocked":
                            if ordinalId > maxUnlockOrdId:
                                maxUnlockOrdId = ordinalId
                                ret = str(row[idxLastCommDate])

                    except Exception as e:
                        logger.error("Error processing %s %s", depid, str(e))

        else:
            logger.debug("Deposition %s empty message file", depid)
            return None

        return ret

    def getLastValidation(self, depid):
        """Returns (datetime, major issue)  of the last time a validation was sent as determined by
        presence of attached validation pdf and parsing message.
        """

        logger.info("Starting for deposition %s", depid)

        msgfile = self.__pI.getFilePath(depid, contentType="messages-to-depositor", formatType="pdbx", fileSource="archive", versionId="1")

        if not os.path.exists(msgfile):
            return (None, None)

        ret = None
        major = None

        myContainerList = []
        ifh = open(msgfile, "r")
        pRd = PdbxReader(ifh)
        pRd.read(myContainerList)

        if len(myContainerList) >= 1:
            c0 = myContainerList[0]

            catObj = c0.getObj("pdbx_deposition_message_file_reference")
            if catObj is None:
                logger.debug("Deposition %s no pdbx_deposition_message_file_reference category", depid)
                return (None, None)

            # Get list of msgids of validtion report
            msgids = set()

            #
            # Get column name index.
            #
            itDict = {}
            itNameList = catObj.getItemNameList()
            for idxIt, itName in enumerate(itNameList):
                itDict[str(itName).lower()] = idxIt

            idxMsgId = itDict["_pdbx_deposition_message_file_reference.message_id"]
            idxContentType = itDict["_pdbx_deposition_message_file_reference.content_type"]

            for row in catObj.getRowList():
                try:
                    msgId = row[idxMsgId]
                    cT = row[idxContentType]

                    if "validation-report-annotate" in cT:
                        msgids.add(msgId)

                except Exception as e:
                    logger.exception("Error processing %s %s", depid, str(e))

            if len(msgids) == 0:
                return (None, None)

            logger.debug("Message ids: %s", msgids)

            # Now check messages

            catObj = c0.getObj("pdbx_deposition_message_info")
            if catObj is None:
                logger.debug("Deposition %s no pdbx_deposition_message_info category", depid)
                return (None, None)

            #
            # Get column name index.
            #
            itDict = {}
            itNameList = catObj.getItemNameList()
            for idxIt, itName in enumerate(itNameList):
                itDict[str(itName).lower()] = idxIt
            #
            idxTimeStamp = itDict["_pdbx_deposition_message_info.timestamp"]
            idxMsgId = itDict["_pdbx_deposition_message_info.message_id"]
            idxMsgText = itDict["_pdbx_deposition_message_info.message_text"]
            idxSendStatus = itDict["_pdbx_deposition_message_info.send_status"]

            lastvalid = None

            for row in catObj.getRowList():
                try:
                    timeStamp = datetime.datetime.strptime(str(row[idxTimeStamp]), "%Y-%m-%d %H:%M:%S")
                    msgId = str(row[idxMsgId])
                    status = str(row[idxSendStatus])
                    msgText = str(row[idxMsgText])

                    if status == "Y" and msgId in msgids:
                        if lastvalid:
                            if timeStamp > lastvalid:
                                # logger.debug("Updating lastvalid %s %s", lastvalid, timeStamp)
                                lastvalid = timeStamp
                                major = self._majorValidation(msgText)

                        else:
                            lastvalid = timeStamp
                            major = self._majorValidation(msgText)

                except Exception as e:
                    logger.exception("Error processing %s %s", depid, str(e))

            ret = lastvalid

        else:
            logger.debug("Deposition %s empty message file", depid)
            ret = None

        logger.info("Returning (%s, %s)", ret, major)

        return (ret, major)

    def _majorValidation(self, msgText):
        """Returns true if there appears to be a major error in the validation test - otherwise False"""

        ret = False
        if re.search("Some major issues", msgText) is not None:
            ret = True

        # logger.debug("Major validation %s", ret)
        return ret

    def getLastSentMessageDate(self, depid, msgtodepositor):
        """Returns datetime  of the ast message sent.  Will return None if no messages sent.
        msgtodepositor is boolean indicating which direction message sent
        """

        logger.info("Starting for deposition %s msgtodepositor %s", depid, msgtodepositor)

        if msgtodepositor:
            msg_content = "messages-to-depositor"
        else:
            msg_content = "messages-from-depositor"

        msgfile = self.__pI.getFilePath(depid, contentType=msg_content, formatType="pdbx", fileSource="archive", versionId="1")

        if not os.path.exists(msgfile):
            return None

        ret = None

        myContainerList = []
        ifh = open(msgfile, "r")
        pRd = PdbxReader(ifh)
        pRd.read(myContainerList)

        if len(myContainerList) >= 1:
            c0 = myContainerList[0]

            # Now check messages

            catObj = c0.getObj("pdbx_deposition_message_info")
            if catObj is None:
                logger.debug("Deposition %s no pdbx_deposition_message_info category", depid)
                return None

            #
            # Get column name index.
            #
            itDict = {}
            itNameList = catObj.getItemNameList()
            for idxIt, itName in enumerate(itNameList):
                itDict[str(itName).lower()] = idxIt
            #
            idxTimeStamp = itDict["_pdbx_deposition_message_info.timestamp"]
            idxSendStatus = itDict["_pdbx_deposition_message_info.send_status"]

            lastsent = None

            for row in catObj.getRowList():
                try:
                    timeStamp = datetime.datetime.strptime(str(row[idxTimeStamp]), "%Y-%m-%d %H:%M:%S")
                    status = str(row[idxSendStatus])

                    if status == "Y":
                        if lastsent:
                            if timeStamp > lastsent:
                                # logger.debug("Updating lastvalid %s %s", lastsent, timeStamp)
                                lastsent = timeStamp
                        else:
                            lastsent = timeStamp

                except Exception as e:
                    logger.exception("Error processing %s %s", depid, str(e))

            ret = lastsent

        else:
            logger.debug("Deposition %s empty message file", depid)
            ret = None

        logger.info("Returning %s", ret)

        return ret



##
# File:  MessagingWebApp.py
# Date:  26-Apr-2012
# Updates:
#
# 2012-04-26    RPS    Ported from prior XXXWebApp implementation template.
# 2013-09-03    RPS    Updated to use cif-based backend storage. Augmented functionality for sending/reading messages.
# 2013-09-13    RPS    added _checkAvailFiles() so that "Associate Files" checkboxes now only appear in UI for files that actually are available on serverside.
#                            Support for displaying confirmation of files referenced when displaying a message in UI.
# 2013-10-04    RPS    Providing file upload input on Compose Message, to accommodate attachment of auxiliary file.
# 2013-10-30    RPS    Support for "Notes" UI and for UI feature for classifying messages (e.g. "action required" or "unread")
# 2013-11-19    RPS    Correcting notification and attachment behavior for scenario when drafts converted to actual sent msgs
# 2014-01-14    RPS    Updates to support notifications to WFM for global "action required" and "read/unread" message status.
# 2014-01-24    RPS    Corrected so that default value for action required in markMsgAsRead is 'Y'.
# 2014-03-04    RPS    Updates to allow tracking of instances wherein new "-annotate" version of cif model file is being propagated to Deposition side.
#                            and also allow tracking of instances wherein validation reports are being propagated to Deposition side.
# 2014-03-21    RPS    Interim fix to prevent cif parsing errors due to unmatched ';'
# 2014-03-17    RPS    Support for UTF-8 capture/persistence into cif files.
# 2014-04-08    RPS    Fix to allow association of messages with uploaded auxiliary files with Common Tool format <fileName>.<extension>.<versionNum>
# 2014-06-09    RPS    Now accommodating up to 2 additional auxiliary file references. And aborting submission of new message when problem arises
#                        with handling of file references.
# 2014-10-16    RPS    Added explicit error messages for cases where annotator "attaches" file with unfamiliar extension.
# 2014-12-04    RPS    Updates for: "archive" of messages, new "Complete Correspondence History" view, tagging messages "for Release", checking for
#                        presence of notes.
# 2014-12-10    RPS    checkGlobalMsgStatus() updated so that can distinguish notes actually authored by annotators from those notes that result from archived comms.
# 2014-12-18    RPS    updated to accommodate handling of automated "reminder" notifications
# 2014-12-23    RPS    short-term fix to eliminate display of '\xa0' in message text.
# 2015-01-29    RPS    Added functionality to allow annotator to disable/enable '*' indicator shown in WFM UI that signals presence of annotator-authored notes.
# 2015-03-02    RPS    Updates per introduction of sanity check safeguards on writes to messaging cif data files.
# 2015-07-28    RPS    Introduced support for "Unlock Dep UI" button and for flagging note/comms from BMRB.
# 2015-12-02    RPS    Updates to optimize response time to user by running template processing in background.
# 2016-02-03    RPS    Calling checkGlobalMsgStatus() after a message requiring "notes" flag is archived by email handler, so that flag is properly refreshed for display in WFM UI
# 2016-02-17    RPS    Introducing use of Message model class to increase encapsulation, improve code organization.
# 2016-08-09    RPS    Introducing support for standalone correspondence viewer. Providing means for detecting presence of notes archived via BMRB emails.
# 2016-09-07    RPS    Providing separate, dedicated means for activating/deactivating UI flag for detecting presence of
#                      notes archived via BMRB emails (vs. standard annotator authored notes).
# 2016-09-14    ZF     Added __checkGroupDeposition() function to support for group deposition
##
"""
wwPDB Messaging web request and response processing modules.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2012 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Raul Sala"
__email__ = "rsala@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import ntpath
import os
import sys
import time
import traceback

try:
    from html import unescape
except ImportError:
    from HTMLParser import HTMLParser

from wwpdb.utils.session.WebRequest import InputRequest, ResponseContent
from wwpdb.apps.msgmodule.depict.MessagingDepict import MessagingDepict
from wwpdb.apps.msgmodule.io.MessagingIo import MessagingIo
from wwpdb.utils.wf.dbapi.StatusDbApi import StatusDbApi
from wwpdb.apps.msgmodule.models.Message import Message

#
# from wwpdb.apps.msgmodule.utils.WfTracking              import WfTracking
#
from wwpdb.utils.config.ConfigInfo import ConfigInfo

#
from wwpdb.utils.wf.dbapi.WfDbApi import WfDbApi

#
from wwpdb.apps.wf_engine.engine.WFEapplications import getdepUIPassword

#
from wwpdb.io.locator.PathInfo import PathInfo

import logging

logger = logging.getLogger(__name__)


class MessagingWebApp(object):
    """Handle request and response object processing for the wwPDB messaging tool application."""

    def __init__(self, parameterDict=None, verbose=False, log=sys.stderr, siteId="WWPDB_DEV"):
        """
        Create an instance of `MessagingWebApp` to manage an messaging web request.

         :param `parameterDict`: dictionary storing parameter information from the web request.
             Storage model for GET and POST parameter data is a dictionary of lists.
         :param `verbose`:  boolean flag to activate verbose logging.
         :param `log`:      stream for logging.

        """
        if parameterDict is None:
            parameterDict = {}
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        self.__siteId = siteId
        self.__cI = ConfigInfo(self.__siteId)
        self.__topPath = self.__cI.get("SITE_WEB_APPS_TOP_PATH")
        self.__topSessionPath = self.__cI.get("SITE_WEB_APPS_TOP_SESSIONS_PATH")
        self.__templatePath = os.path.join(self.__topPath, "htdocs", "msgmodule")
        #

        if isinstance(parameterDict, dict):
            self.__myParameterDict = parameterDict
        else:
            self.__myParameterDict = {}

        if self.__verbose:
            logger.info("+MessagingWebApp.__init() - REQUEST STARTING ------------------------------------")
            logger.info("+MessagingWebApp.__init() - dumping input parameter dictionary")
            logger.info("%s", "".join(self.__dumpRequest()))

        self.__reqObj = InputRequest(self.__myParameterDict, verbose=self.__verbose, log=self.__lfh)
        #
        self.__reqObj.setValue("TopSessionPath", self.__topSessionPath)
        self.__reqObj.setValue("TemplatePath", self.__templatePath)
        self.__reqObj.setValue("TopPath", self.__topPath)
        self.__reqObj.setValue("WWPDB_SITE_ID", self.__siteId)
        os.environ["WWPDB_SITE_ID"] = self.__siteId
        #
        self.__reqObj.setDefaultReturnFormat(return_format="html")
        #
        if self.__verbose:
            logger.info("-----------------------------------------------------")
            logger.info("+MessagingWebApp.__init() Leaving _init with request contents")
            logger.info("%s", str(self.__reqObj))
            logger.info("---------------MessagingWebApp - done -------------------------------")

    def doOp(self):
        """Execute request and package results in response dictionary.

        :Returns:
             A dictionary containing response data for the input request.
             Minimally, the content of this dictionary will include the
             keys: CONTENT_TYPE and REQUEST_STRING.
        """
        stw = MessagingWebAppWorker(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC = stw.doOp()
        if self.__debug:
            rqp = self.__reqObj.getRequestPath()
            logger.debug("+MessagingWebApp.doOp() operation %s", rqp)
            logger.debug("+MessagingWebApp.doOp() return format %s", self.__reqObj.getReturnFormat())
            if rC is not None:
                logger.debug("%s", "".join(rC.dump()))
            else:
                logger.debug("+MessagingWebApp.doOp() return object is empty")

        #
        # Package return according to the request return_format -
        #
        return rC.get()

    def __dumpRequest(self):
        """Utility method to format the contents of the internal parameter dictionary
        containing data from the input web request.

        :Returns:
            ``list`` of formatted text lines
        """
        retL = []
        retL.append("\n-----------------MessagingWebApp().__dumpRequest()-----------------------------\n")
        retL.append("Parameter dictionary length = %d\n" % len(self.__myParameterDict))
        for k, vL in self.__myParameterDict.items():
            retL.append("Parameter %30r :" % k)
            for v in vL:
                retL.append(" ->  %r\n" % v)
        retL.append("-------------------------------------------------------------\n")
        return retL


class MessagingWebAppWorker(object):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        Worker methods for the wwPDB messaging application

        Performs URL - application mapping and application launching
        for wwPDB messaging tool.

        All operations can be driven from this interface which can
        supplied with control information from web application request
        or from a testing application.
        """
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = True
        self.__reqObj = reqObj
        self.__sObj = None
        self.__sessionId = None
        self.__sessionPath = None
        # self.__rltvSessionPath = None
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        # self.__cI = ConfigInfo(self.__siteId)
        #
        # Added by ZF
        #
        self.__checkGroupDeposition()
        #
        self.__appPathD = {
            "/service/messaging/environment/dump": "_dumpOp",
            "/service/messaging/launch": "_launchOp",
            "/service/messaging/get_dtbl_data": "_getDataTblData",
            "/service/messaging/get_dtbl_config_dtls": "_getDataTblConfigDtls",
            "/service/messaging/mark_msg_read": "_markMsgAsRead",
            "/service/messaging/tag_msg": "_tagMsg",
            "/service/messaging/submit_msg": "_submitMsg",
            "/service/messaging/update_draft_state": "_updateDraftState",
            # "/service/messaging/delete_row": "_deleteRowOp",  # Not implemented on client or server properly
            # "/service/messaging/test_see_json": "_getDataTblDataRawJsonOp",  # Disable for now
            "/service/messaging/exit_not_finished": "_exit_notFinished",
            "/service/messaging/exit_finished": "_exit_finished",
            "/service/messaging/get_msg": "_getMsg",
            "/service/messaging/check_avail_files": "_checkAvailFiles",
            "/service/messaging/get_files_rfrncd": "_getFilesRfrncd",
            "/service/messaging/check_global_msg_status": "_checkGlobalMsgStatus",
            "/service/messaging/archive_msg": "_archiveMsg",
            "/service/messaging/forward_msg": "_forwardMsg",
            "/service/messaging/display_msg": "_displayMsg",
            "/service/messaging/view_correspondence": "_viewCorrespondence",
            "/service/messaging/verify_depid": "_verifyDepositionId",
            "/service/messaging/toggle_notes_flagging": "_toggleNotesFlaggingOp",
            "/service/messaging/get_depui_pwd": "_getDepUiPwd",
            "/service/messaging/get_msg_tmplts": "_getMsgTmplts",
            # ##############  below are URLs to be used for WFM environ######################
            "/service/messaging/new_session/wf": "_launchOp",
            "/service/messaging/new_session/": "_launchOp",
            "/service/messaging/new_session": "_launchOp",
            "/service/messaging/wf/test": "_wfDoSomethingOp",
            "/service/messaging/wf/launch": "_wfLaunchOp",
            "/service/messaging/wf/exit_not_finished": "_exit_notFinished",
            "/service/messaging/wf/exit_finished": "_exit_finished"
            # ##################################################################################################
        }

    def doOp(self):
        """Map operation to path and invoke operation.

        :Returns:

        Operation output is packaged in a ResponseContent() object.
        """
        return self.__doOpException()

    # def __doOpNoException(self):
    #     """Map operation to path and invoke operation.  No exception handling is performed.

    #     :Returns:

    #     Operation output is packaged in a ResponseContent() object.
    #     """
    #     #
    #     reqPath = self.__reqObj.getRequestPath()
    #     if reqPath not in self.__appPathD:
    #         # bail out if operation is unknown -
    #         rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
    #         rC.setError(errMsg="Unknown operation")
    #         return rC
    #     else:
    #         mth = getattr(self, self.__appPathD[reqPath], None)
    #         rC = mth()
    #     return rC

    def __doOpException(self):
        """Map operation to path and invoke operation.  Exceptions are caught within this method.

        :Returns:

        Operation output is packaged in a ResponseContent() object.
        """
        #
        try:
            reqPath = self.__reqObj.getRequestPath()
            if reqPath not in self.__appPathD:
                # bail out if operation is unknown -
                rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
                rC.setError(errMsg="Unknown operation")
            else:
                mth = getattr(self, self.__appPathD[reqPath], None)
                rC = mth()
            return rC
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In processing doOpException")
            rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            rC.setError(errMsg="Operation failure")
            return rC

    ################################################################################################################
    # ------------------------------------------------------------------------------------------------------------
    #      Top-level REST methods
    # ------------------------------------------------------------------------------------------------------------
    #
    def _dumpOp(self):
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setHtmlList(self.__reqObj.dump(format="html"))
        return rC

    def _launchOp(self):
        """Launch wwPDB messaging module interface

        :Helpers:
            wwpdb.apps.msgmodule.depict.MessagingDepict

        :Returns:
            Operation output is packaged in a ResponseContent() object.
            The output consists of a HTML starter container page for quicker return to the client.
            This container page is then populated with content via AJAX calls.
        """
        if self.__verbose:
            logger.info("+MessagingWebAppWorker._launchOp() Starting now")
        # determine if currently operating in Workflow Managed environment
        bIsWorkflow = self.__isWorkflow()
        #
        self.__getSession()
        #
        fileSource = str(self.__reqObj.getValue("filesource")).strip().lower()
        # depId = str(self.__reqObj.getValue("identifier")).upper()
        #
        self.__reqObj.setDefaultReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        if self.__verbose:
            logger.info("+MessagingWebAppWorker._launchOp() workflow flag is %r", bIsWorkflow)

        if bIsWorkflow:
            # Update WF status database --
            pass
            # bSuccess = self.__updateWfTrackingDb("open")
            # if( not bSuccess ):
            #     rC.setError(errMsg="+MessagingWebAppWorker._launchOp() - TRACKING status, update to 'open' failed for session %s \n" % self.__sessionId )
            # else:
            #     if (self.__verbose):
            #         self.__lfh.write("+MessagingWebAppWorker._launchOp() Tracking status set to open\n")
        else:
            if fileSource and fileSource == "rcsb_dev":
                pass
        #
        if self.__verbose:
            logger.info("+MessagingWebAppWorker._launchOp() Call MessagingDepict with workflow %r", bIsWorkflow)
        #
        msgngDpct = MessagingDepict(self.__verbose, self.__lfh)
        msgngDpct.setSessionPaths(self.__reqObj)
        oL = msgngDpct.doRender(self.__reqObj, bIsWorkflow)
        rC.setHtmlText("\n".join(oL))
        #
        return rC

    def _verifyDepositionId(self):

        #
        rtrnDict = {}
        #
        depId = str(self.__reqObj.getValue("identifier")).upper()
        #
        if self.__verbose:
            logger.info(" -- Starting.")

        self.__getSession()
        #
        if self.__verbose:
            logger.info("-- dep_id is: %s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        archivePth = pI.getArchivePath(depId)
        #
        if self.__verbose:
            logger.info(" -- archivePth is: %s", archivePth)
        if self.__verbose:
            logger.info(" -- os.access(archivePth,os.F_OK) is: %s", os.access(archivePth, os.F_OK))
        #
        rtrnDict["found"] = "y" if (os.access(archivePth, os.F_OK)) else "n"

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _viewCorrespondence(self):
        """Launch wwPDB messaging module interface -- "View All Correspondence" view

        :Helpers:
            wwpdb.apps.msgmodule.depict.MessagingDepict

        :Returns:
            Operation output is packaged in a ResponseContent() object.
            The output consists of a HTML starter container page for quicker return to the client.
            This container page is then populated with content via AJAX calls.
        """
        #
        if self.__verbose:
            logger.info("Starting now")
        self.__getSession()
        #
        self.__reqObj.setDefaultReturnFormat(return_format="html")
        self.__reqObj.setValue("filesource", "archive")  # setting here for downstream processing
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgngDpct = MessagingDepict(self.__verbose, self.__lfh)
        msgngDpct.setSessionPaths(self.__reqObj)
        oL = msgngDpct.doRenderAllCorrespondence(self.__reqObj)
        rC.setHtmlText("\n".join(oL))
        #
        return rC

    def _getDataTblConfigDtls(self):
        """Get config details for staging display of messages for given deposition dataset in webpage.
        Data returned includes HTML <table> starter template for displaying messages
        and various settings for column, cell specific display details, validation behavior, etc.

        :Helpers:
            wwpdb.apps.msgmodule.depict.MessagingDepict

        :Returns:
            Operation output is packaged in a ResponseContent() object.
            The output consists of JSON object which has two primary properties:
                'html' --> <table> template representing the category with column headers for
                            each attribute defined for the category and conforming to structure
                            expected by jQuery DataTables plugin
                'dtbl_config_dict' --> multi-layered dictionary of display/validation settings to be used
                                 for configuring the behavior of the DataTable
        """
        #
        rtrnDict = {}
        dtblConfigDict = {}
        #
        if self.__verbose:
            logger.info("-- Starting.")

        self.__getSession()

        depId = self.__reqObj.getValue("identifier")
        if self.__verbose:
            logger.info(" -- dep_id is:%s", depId)
        #
        sendStatus = self.__reqObj.getValue("send_status")
        #
        sUseThreadedRsltSet = self.__reqObj.getValue("usethreaded")
        bUseThreadedRsltSet = False
        #
        if sUseThreadedRsltSet and sUseThreadedRsltSet == "true":
            bUseThreadedRsltSet = True
            #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        rsltSetDict = msgingIo.getMsgRowList(
            p_depDataSetId=depId, p_sSendStatus=sendStatus, p_bServerSide=False, p_bThreadedRslts=bUseThreadedRsltSet
        )  # serverside False b/c only getting indentation info necessary for initial config
        #
        msgingDpct = MessagingDepict(verbose=self.__verbose, log=self.__lfh)
        dataTblTmplt, dtblConfigDict = msgingDpct.getDataTableTemplate(self.__reqObj)
        # dataTblTmplt = msgngDpct.getDataTableTemplate( depId, msgColList, bTrnspsdTbl, catObjDict['col_displ_name'] )  # example of supporting user friendly column display names
        #
        if "CURRENT_NUM_MSGS_TO_DPSTR" in rsltSetDict:
            dtblConfigDict["CURRENT_NUM_MSGS_TO_DPSTR"] = rsltSetDict[
                "CURRENT_NUM_MSGS_TO_DPSTR"
            ]  # adding member representing total #records of msgs to depositor (live AND draft)
        if "CURRENT_NUM_NOTES" in rsltSetDict:
            dtblConfigDict["CURRENT_NUM_NOTES"] = rsltSetDict["CURRENT_NUM_NOTES"]  # adding member representing total #records of notes (live AND draft)
        #
        if bUseThreadedRsltSet and "INDENT_DICT" in rsltSetDict:
            dtblConfigDict["INDENT_DICT"] = rsltSetDict["INDENT_DICT"]  # adding member representing threading/indent-formatting defined per msgid
        #
        rtrnDict["html"] = "".join(dataTblTmplt)

        # stuff config dict into return dictionary for return to web page
        rtrnDict["dtbl_config_dict"] = dtblConfigDict

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _getDataTblData(self):
        """Get data needed to populate DataTable for displaying messages for given deposition dataset ID

        :Helpers:
            wwpdb.apps.msgmodule.depict.MessagingDepict
            wwpdb.apps.msgmodule.io.MessagingIo

        :Returns:
            Operation output is packaged in a ResponseContent() object.
            The output consists of a JSON object representing the category/items
            and conforming to structure expected by jQuery DataTables plugin for rendering
        """
        #
        self.__getSession()
        # sUseServerSide = str( self.__reqObj.getValue("serverside") )
        depId = self.__reqObj.getValue("identifier")
        sendStatus = self.__reqObj.getValue("send_status")
        sUseThreadedRsltSet = self.__reqObj.getValue("usethreaded")
        sContentType = self.__reqObj.getValue("content_type")
        bCommHstryRqstd = True if sContentType == "commhstry" else False
        #
        if sUseThreadedRsltSet and sUseThreadedRsltSet == "true":
            bUseThreadedRsltSet = True
        else:
            bUseThreadedRsltSet = False
        #
        rsltSetDict = {}
        #
        if self.__verbose:
            logger.info(" -- dep_id is: %s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        _bOk, msgColList = msgingIo.getMsgColList(bCommHstryRqstd)
        #
        # if( sUseServerSide == 'true' ):
        # DataTables related query string params:
        iDisplayStart = int(self.__reqObj.getValue("iDisplayStart"))
        iDisplayLength = int(self.__reqObj.getValue("iDisplayLength"))
        sEcho = int(self.__reqObj.getValue("sEcho"))  # casting to int as recommended by DataTables

        ##################################################################
        # we need to accommodate any search filtering taking place
        # any search strings provided are processed by MsgingIo against
        # resultset of data that was encoded in ascii on its way into cif datafile
        # and then decoded back to UTF8 for return/display in browser.
        # so we must mimic the same process for any search terms before we test
        # such terms for potential matches with the data in the resultset.
        # this is why self.__encodeForSearching() is invoked
        ##################################################################

        # handling for single global search term
        sSearch = self.__reqObj.getRawValue("sSearch")
        sSearch = self.__encodeForSearching(sSearch)
        logger.info("-- sSearch is: %r", sSearch)

        # ############# in below block we are accommodating any requests for column-specific search filtering ###################################
        numColumns = len(msgColList)
        colSearchDict = {}
        for n in range(0, numColumns):
            qryStrParam = "sSearch_" + str(n)
            qryBoolParam = "bSearchable_" + str(n)

            bIsColSearchable = self.__reqObj.getValue(qryBoolParam) == "true"
            if bIsColSearchable:
                srchString = self.__reqObj.getValue(qryStrParam)
                if srchString and len(srchString) > 1:
                    colSearchDict[n] = self.__encodeForSearching(srchString)
                    if self.__verbose and self.__debug:
                        logger.debug(" -- search term for field[%s] is: %s", n, colSearchDict[n])
        ################################################################################################################################
        rsltSetDict = msgingIo.getMsgRowList(
            p_depDataSetId=depId,
            p_sSendStatus=sendStatus,
            p_bServerSide=True,
            p_iDisplayStart=iDisplayStart,
            p_iDisplayLength=iDisplayLength,
            p_sSrchFltr=sSearch,
            p_colSearchDict=colSearchDict,
            p_bThreadedRslts=bUseThreadedRsltSet,
        )
        msgRecordList = rsltSetDict["RECORD_LIST"]
        # """
        # else: # 2014-09-10, RPS: non-serverside processing is only being used in _getDataTblConfigDtls() -- consider removing?
        #     rsltSetDict = msgingIo.getMsgRowList( p_depDataSetId=depId, p_sSendStatus=sendStatus, p_bServerSide=False,  p_bThreadedRslts=bUseThreadedRsltSet )
        #     msgRecordList = rsltSetDict['RECORD_LIST']
        #     iDisplayStart = 0 #always getting full resultset when not using server=side processing so setting iDisplayStart explicitly to 0
        # """
        #
        msgngDpct = MessagingDepict(verbose=self.__verbose, log=self.__lfh)
        dataTblDict = msgngDpct.getJsonDataTable(msgRecordList, msgColList, iDisplayStart)
        # if( sUseServerSide == 'true' ):
        dataTblDict["sEcho"] = sEcho
        dataTblDict["iTotalRecords"] = rsltSetDict["TOTAL_RECORDS"]
        dataTblDict["iTotalDisplayRecords"] = rsltSetDict["TOTAL_DISPLAY_RECORDS"]
        #
        rC.addDictionaryItems(dataTblDict)

        return rC

    def _getDataTblDataRawJsonOp(self):
        """for DEV -- return payload input to DataTables to be displayed on webpage as JSON object for inspection"""

        if self.__verbose:
            logger.info("+MessagingWebAppWorker._getDataTblDataRawJsonOp() starting\n")

        self.__getSession()
        # iDisplayStart = int( self.__reqObj.getValue("iDisplayStart") )
        # iDisplayLength = int( self.__reqObj.getValue("iDisplayLength") )
        # sEcho = int( self.__reqObj.getValue("sEcho") ) # casting to int as recommended by DataTables
        sEcho = 10

        depId = self.__reqObj.getValue("identifier")
        #
        if self.__verbose:
            logger.info("-- dep_id is:%s", depId)
        #
        self.__reqObj.setDefaultReturnFormat(return_format="html")

        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)

        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        _bOk, msgColList = msgingIo.getMsgColList()
        msgRecordList, iTotalRecords, iTotalDisplayRecords = msgingIo.getMsgRowList(depId, True, 0, 20, "")
        #
        msgngDpct = MessagingDepict(verbose=self.__verbose, log=self.__lfh)
        dataTblDict = msgngDpct.getJsonDataTable(msgRecordList, msgColList, 0)
        dataTblDict["sEcho"] = sEcho
        dataTblDict["iTotalRecords"] = iTotalRecords
        dataTblDict["iTotalDisplayRecords"] = iTotalDisplayRecords

        rC.setHtmlText(str(dataTblDict))

        return rC

    def _getMsgTmplts(self):
        """Get

        :Helpers:

        :Returns:

        """
        #
        rtrnDict = {}
        #
        if self.__verbose:
            logger.info(" -- Starting.")
            #
        self.__getSession()
        #
        bIsWorkflow = self.__isWorkflow()
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        bIsWorkflow = self.__isWorkflow()
        msgngDpct = MessagingDepict(self.__verbose, self.__lfh)
        msgngDpct.setSessionPaths(self.__reqObj)
        oL = msgngDpct.getMsgTmplts(self.__reqObj, bIsWorkflow)

        rtrnDict["html"] = "\n".join(oL)

        statusCode = self.__reqObj.getValueOrDefault("status_code", default=None)
        if statusCode:
            rtrnDict["status_code"] = statusCode

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _checkGlobalMsgStatus(self):
        """Method serves to check and register/communicate latest collective messaging status conditions
        (i.e. conditions that refer not to any particular message but to the entire collection of messages)
        e.g. are there any unread messages?, are there any messages requiring action?, any messages associated with release?
        NOTE: method also invokes call to update database with latest condition flags.

        :Returns: JSON returned to the browser containing relevant name/value pairs for various conditions

        """
        #
        rtrnDict = {}
        bAllMsgsRead = False
        bAllMsgsActioned = False
        bAnyFlagsForRelease = False
        #
        if self.__verbose:
            logger.info("Starting")

        self.__getSession()
        #
        depId = str(self.__reqObj.getValue("identifier"))
        #
        if self.__verbose:
            logger.info("dep_id is:%s", depId)
        #
        activateNotesFlagging = self._getNotesFlaggingStatus()
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        bAllMsgsRead = self.__checkAllMsgsRead()
        bAllMsgsActioned = self.__checkAllMsgsActioned()
        bAnyFlagsForRelease = self.__checkAnyReleaseFlags()
        bAnyNotesIncldngArchvdMsgs, bAnnotNotes, bBmrbNotes, iNumNotesRecords = self.__checkAnyNotesExist()

        # logger.info(("+%s.%s() -- bAnyNotesIncldngArchvdMsgs is '%s' -- bAnnotNotes is '%s' -- iNumNotesRecords is '%s' for DEPID %s \n" % (className, methodName,bAnyNotesIncldngArchvdMsgs,bAnnotNotes,iNumNotesRecords,depId))  # noqa: E501

        if bAllMsgsRead is True:
            rtrnDict["all_msgs_read"] = "true"
            newMsgsFlag = ""
        else:
            rtrnDict["all_msgs_read"] = "false"
            newMsgsFlag = "N"  # i.e. "N"ew message(s) present

        if bAllMsgsActioned is True:
            rtrnDict["all_msgs_actioned"] = "true"
            msgsNeedActionFlag = ""
        else:
            rtrnDict["all_msgs_actioned"] = "false"
            msgsNeedActionFlag = "T"  # i.e. "T"odo action(s) present

        if bAnyFlagsForRelease is True:
            rtrnDict["any_msgs_for_release"] = "true"
            msgsForReleaseFlag = "R"  # i.e. for "R"elease
        else:
            rtrnDict["any_msgs_for_release"] = "false"
            msgsForReleaseFlag = ""  # i.e. for "R"elease

        if bAnnotNotes is True:
            notesExistFlag = "*"
        else:
            notesExistFlag = ""

        if bBmrbNotes is True:
            bmrbNotesExistFlag = "B"
        else:
            bmrbNotesExistFlag = ""

        if bAnyNotesIncldngArchvdMsgs is True:
            rtrnDict["any_notes_exist"] = "true"
        else:
            rtrnDict["any_notes_exist"] = "false"

        rtrnDict["num_notes_records"] = iNumNotesRecords

        aggregateFlag = newMsgsFlag + msgsNeedActionFlag + msgsForReleaseFlag
        #
        if activateNotesFlagging == "true":
            aggregateFlag += notesExistFlag
        #
        rtrnDict["notes_flag_active"] = activateNotesFlagging

        if bBmrbNotes is True:
            # only send this key/value pair back if there are actually BMRB notes present, otherwise UI has no need to know
            activateNotesFlaggingBmrb = self._getNotesFlaggingStatus("bmrb")
            rtrnDict["notes_flag_active_bmrb"] = activateNotesFlaggingBmrb

            if activateNotesFlaggingBmrb == "true":
                aggregateFlag += bmrbNotesExistFlag

        #
        bSuccess = self._updateWfNotifyStatus(depId, aggregateFlag)
        if self.__verbose and bSuccess:
            logger.info("-- NOTIFY status updated to '%s' for DEPID %s", aggregateFlag, depId)

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _getNotesFlaggingStatus(self, subtype=""):
        """Gets flagging status for notes content
        Relies on simple mechanism of having a file containing "true" or "false" value,
        indicating whether or not visual flag (icon in user interface) for notes content should be activate

        :param `subtype`:    optional parameter indicating subtype of Notes being handled for current request (e.g. "bmrb" notes)

        :Returns: value of current flagging status

        """
        #
        if self.__verbose:
            logger.info(" -- Starting.")
            #
        activateNotesFlagging = "true"  # default
        depId = str(self.__reqObj.getValue("identifier"))
        #
        if self.__verbose:
            logger.info("-- depId is: %s", depId)
            logger.info("+-- notes subtype: %s", subtype)
        #
        notesFlaggingStatusFilePathAbs = self.__getNotesFlaggingStatusFilePath(subtype=subtype)
        #
        if os.access(notesFlaggingStatusFilePathAbs, os.F_OK):
            try:
                fp = open(notesFlaggingStatusFilePathAbs, "r")
                lines = fp.readlines()
                fp.close()
                activateNotesFlagging = lines[0][:-1]
            except:  # noqa: E722 pylint: disable=bare-except
                if self.__verbose:
                    logger.info(" -- problem reading notesFlaggingStatusFilePathAbs at:%s", notesFlaggingStatusFilePathAbs)
        else:
            # file doesn't exist yet which is true if this is first time annotator is accessing messaging UI for the given depID
            # so we create the file and set default status of "ON"
            fp = open(notesFlaggingStatusFilePathAbs, "w")
            fp.write("%s\n" % "true")
            fp.close()
            #
        return activateNotesFlagging

    def _toggleNotesFlaggingOp(self):
        """Toggles flagging status for notest content

        :Returns:
            Operation output is packaged in a ResponseContent() object.
        """
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        fstatus = self._toggleNotesFlagging()
        rC.setText(fstatus)
        return rC

    def _toggleNotesFlagging(self, subtype=""):
        """Toggles flagging status for notes content
        Relies on simple mechanism of having a file containing "true" or "false" value,
        indicating whether or not visual flag (icon in user interface) for notes content should be activate

        :param `subtype`:    optional parameter indicating subtype of Notes being handled for current request (e.g. "bmrb" notes)

        :Returns: value of new flagging status is echoed back to calling code

        """
        depId = str(self.__reqObj.getValue("identifier"))
        subtypeFromBrwser = str(self.__reqObj.getValue("subtype"))
        #
        if subtypeFromBrwser and len(subtypeFromBrwser) > 1:
            subTypeRqstd = subtypeFromBrwser
        else:
            subTypeRqstd = subtype

        if self.__verbose:
            logger.info("-- Starting.")
            logger.info("-- depId: %s", depId)
            logger.info("-- notes subTypeRqstd: %s", subTypeRqstd)
        #
        self.__getSession()
        #
        if len(subTypeRqstd) > 1:
            activateNotesFlagging = str(self.__reqObj.getValue("activate_notes_flagging_" + subTypeRqstd))
        else:
            activateNotesFlagging = str(self.__reqObj.getValue("activate_notes_flagging"))
        #
        if self.__verbose:
            logger.info(" -- desired flag_status is: %s", activateNotesFlagging)
        #
        notesFlaggingStatusFilePathAbs = self.__getNotesFlaggingStatusFilePath(subtype=subTypeRqstd)
        #
        if os.access(notesFlaggingStatusFilePathAbs, os.F_OK):
            try:
                fp = open(notesFlaggingStatusFilePathAbs, "w")
                fp.write("%s\n" % activateNotesFlagging)
                fp.close()
            except:  # noqa: E722 pylint: disable=bare-except
                if self.__verbose:
                    logger.info("-- problem writing to notesFlaggingStatusFilePathAbs at:%s", notesFlaggingStatusFilePathAbs)
        else:
            fp = open(notesFlaggingStatusFilePathAbs, "w")
            fp.write("%s\n" % activateNotesFlagging)
            fp.close()
            #
        return activateNotesFlagging

    def __checkGroupDeposition(self):
        #
        # Added by ZF
        #
        """Check if the depId is belonged to a group deposition"""
        depId = str(self.__reqObj.getValue("identifier")).upper()
        if not depId:
            return
        elif depId.startswith("G_") and len(depId) == 9:
            self.__reqObj.setValue("groupid", depId)
            return
        #
        statusApi = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        groupId = statusApi.getGroupId(depId)
        if groupId:
            self.__reqObj.setValue("groupid", groupId)
        #

    def __getNotesFlaggingStatusFilePath(self, subtype=""):
        """Get

        :Helpers:

        :Returns:

        """
        depId = str(self.__reqObj.getValue("identifier"))
        #
        # Added by ZF
        #
        groupId = str(self.__reqObj.getValue("groupid"))
        #
        if self.__verbose:
            logger.info(" -- Starting.")
            logger.info(" -- depId: %s", depId)
            logger.info(" -- notes subtype: %s", subtype)
        #
        suffix = "_" + subtype.upper() if len(subtype) > 1 else ""
        #
        pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        #
        # Added by ZF
        #
        if groupId:
            archivePth = pI.getArchivePath(groupId)
        else:
            archivePth = pI.getArchivePath(depId)
        #
        notesFlaggingStatusFilePathAbs = os.path.join(archivePth, "NOTES_FLAGGING_STATUS" + suffix)
        #
        if self.__verbose:
            logger.info(" returning notesFlaggingStatusFilePathAbs as:%s", notesFlaggingStatusFilePathAbs)
        #
        return notesFlaggingStatusFilePathAbs

    def __checkAllMsgsRead(self):
        """Get

        :Helpers:

        :Returns:

        """
        #
        bAllMsgsRead = False
        #
        if self.__verbose:
            logger.info("-- Starting")

            #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        bAllMsgsRead = msgingIo.areAllMsgsRead()

        return bAllMsgsRead

    def __checkAllMsgsActioned(self):
        """Get

        :Helpers:

        :Returns:

        """
        #
        bAllMsgsActioned = False
        #
        if self.__verbose:
            logger.info("Starting.")

            #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        bAllMsgsActioned = msgingIo.areAllMsgsActioned()

        return bAllMsgsActioned

    def __checkAnyReleaseFlags(self):
        """Checks whether there are any messages flagged to indicate entry is ready for release.

        :Helpers:

        :Returns:

        """
        bForRelease = False
        #
        if self.__verbose:
            logger.info("-- Starting.")

            #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        bForRelease = msgingIo.anyReleaseFlags()

        return bForRelease

    def __checkAnyNotesExist(self):
        """Get

        :Helpers:

        :Returns:

        """
        #
        if self.__verbose:
            logger.info("-- Starting.")

            #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        return msgingIo.anyNotesExist()

    def _updateWfNotifyStatus(self, p_depId, p_status):
        #
        bSuccess = False
        #
        if self.__verbose:
            logger.info(" Starting.")
        try:
            # wft=WfTracking(verbose=self.__verbose,log=self.__lfh)
            # bSuccess = wft.setDepMsgNotifyStatus(depId=p_depId,status=p_status)
            #
            # Removed by ZF
            #
            # wfApi= WfDbApi(verbose=True)
            # sql = "update deposition set notify = '"+p_status+"' where dep_set_id = '" + p_depId.upper() + "'"
            # wfApi.runUpdateSQL(sql);
            #
            # Added by ZF
            #
            statusApi = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
            groupId = str(self.__reqObj.getValue("groupid"))
            if groupId:
                statusApi.runUpdate(table="batch_user_data", where={"dep_set_id": groupId.upper()}, data={"notify": p_status})
                entryList = statusApi.getEntryIdList(groupId=groupId)
                for entry in entryList:
                    statusApi.runUpdate(table="deposition", where={"dep_set_id": entry}, data={"notify": p_status})
                #
            else:
                statusApi.runUpdate(table="deposition", where={"dep_set_id": p_depId.upper()}, data={"notify": p_status})
            #
            bSuccess = True
        except:  # noqa: E722 pylint: disable=bare-except
            bSuccess = False
            if self.__verbose:
                logger.info(" - TRACKING status, update to '%s' failed for depID %s", p_status, p_depId)
            logger.exception("In _updateWfNotifyStatus")
            #
        return bSuccess

    def _getFilesRfrncd(self):
        """Retrieve list of files referenced by any messages for this dataset ID

        :Helpers:

        :Returns:

        """
        rtrnDict = {}
        #
        if self.__verbose:
            logger.info("Starting.")

        self.__getSession()
        #
        depId = self.__reqObj.getValue("identifier")
        #
        if self.__verbose:
            logger.info("dep_id is:%s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        rtrnDict["files_rfrncd"] = msgingIo.getFilesRfrncd(depId)

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _checkAvailFiles(self):
        """Get

        :Helpers:

        :Returns:

        """
        #
        rtrnDict = {}
        fileLst = []
        #
        if self.__verbose:
            logger.info("Starting")

        self.__getSession()
        #
        depId = self.__reqObj.getValue("identifier")
        #
        if self.__verbose:
            logger.info(" dep_id is:%s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        msgingIo.initializeDataStore()  # THIS CALL MUST BE MADE HERE TO PARSE MODEL FILE AND FILTER
        fileLst = msgingIo.checkAvailFiles(depId)
        #
        rtrnDict["file_list"] = fileLst

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _getMsg(self):
        """Get data for a single message

        :Helpers:
            wwpdb.apps.msgmodule.io.MessagingIo

        :Returns:
            JSON response representing a given message and its attributes

        """
        rtrnDict = {}
        msgDict = {}
        #
        if self.__verbose:
            logger.info(" Starting")

        self.__getSession()
        #
        depId = self.__reqObj.getValue("identifier")
        msgId = self.__reqObj.getValue("msg_id")
        #
        if self.__verbose:
            logger.info(" dep_id is: %s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        msgDict = msgingIo.getMsg(msgId, depId)
        #

        # stuff config dict into return dictionary for return to web page
        rtrnDict["msg_dict"] = msgDict

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _displayMsg(self):
        """Get

        :Helpers:

        :Returns:

        """
        bIsWorkflow = self.__isWorkflow()
        #
        msgDict = {}
        #
        if self.__verbose:
            logger.info("Starting.")

        self.__getSession()
        #
        depId = self.__reqObj.getValue("identifier")
        msgId = self.__reqObj.getValue("msg_id")
        #
        if self.__verbose:
            logger.info("dep_id is:%s", depId)
        #
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        msgDict = msgingIo.getMsg(msgId, depId)
        logger.info("msgDict is:  %r", msgDict)
        #
        if msgDict["parent_message_id"] != msgDict["message_id"]:
            parentMsgDict = msgingIo.getMsg(msgDict["parent_message_id"], depId)
            msgDict["parent_msg_sndr"] = parentMsgDict["sender"]
            msgDict["parent_msg_datetime"] = parentMsgDict["timestamp"]
            msgDict["parent_msg_text"] = parentMsgDict["message_text"]
            msgDict["parent_msg_dsply"] = ""
        else:
            msgDict["parent_msg_sndr"] = ""
            msgDict["parent_msg_datetime"] = ""
            msgDict["parent_msg_text"] = ""
            msgDict["parent_msg_dsply"] = ""
            msgDict["parent_msg_dsply"] = "displaynone"

        msgDict["files_referenced"] = (msgingIo.getFilesRfrncd(depId, msgId)).get(msgId, [])
        msgngDpct = MessagingDepict(self.__verbose, self.__lfh)
        msgngDpct.setSessionPaths(self.__reqObj)
        oL = msgngDpct.doRenderDisplayMsg(self.__reqObj, bIsWorkflow, msgDict)
        rC.setHtmlText("\n".join(oL))
        #
        return rC

    def _archiveMsg(self):
        return self._propagateMsg("archive")

    def _forwardMsg(self):
        return self._propagateMsg("forward")

    def _propagateMsg(self, actionType):
        """
        :Helpers:

        :Returns:

        """
        #
        #
        self.__getSession()
        # depId = self.__reqObj.getValue("entry_id") # getValue("identifier")
        mode = self.__reqObj.getValue("mode")
        if mode and mode == "manual":
            depIdCsvLst = self.__reqObj.getValue("target_identifier")
            origDepId = self.__reqObj.getValue("identifier")
            msgType = actionType + "_manual"
        else:
            depIdCsvLst = self.__reqObj.getValue("identifier")
            origDepId = None
            msgType = actionType + "_auto"

        flagMsg = self.__reqObj.getValue("flag")
        if flagMsg and flagMsg == "y":
            msgType = msgType + "_flag"
        #
        subject = self.__reqObj.getRawValue("subject")
        subject = subject if subject is not None and len(subject) > 1 else "test subject"
        #
        if actionType == "archive":
            subjPrefix = "ARCHIVED: "
        else:
            # subjPrefix = "FWD: " if msgCategory != "reminder" else ""
            subjPrefix = "FWD: "
        #
        # setting below request parameters for downstream processing
        self.__reqObj.setValue("subject", subjPrefix + subject)  # setting here for downstream processing
        self.__reqObj.setValue("send_status", "Y")  # setting here for downstream processing
        self.__reqObj.setValue("filesource", "archive")  # setting here for downstream processing
        self.__reqObj.setValue("message_type", msgType)  # setting here for downstream processing
        self.__reqObj.setValue("orig_identifier", origDepId)  # only used when archive action is via "manual" mode
        contentType = "notes" if (actionType == "archive") else "msgs"
        self.__reqObj.setValue("content_type", contentType)  # setting here for downstream processing (as opposed to "msgs")
        self.__reqObj.setValue("message_state", "livemsg")  # setting here for downstream processing
        #
        rtrnDict = {}
        rtrnDict["success"] = {}
        rtrnDict["success"]["job"] = "error"
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        depIdLst = depIdCsvLst.upper().split(",")
        #
        # Added by ZF
        #
        statusApi = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        for depId in depIdLst:
            self.__reqObj.setValue("identifier", depId)  # setting here for downstream processing
            #
            # Added by ZF
            #
            groupId = ""
            if depId.startswith("G_"):
                groupId = depId
            else:
                found_groupId = statusApi.getGroupId(depId)
                if found_groupId:
                    groupId = found_groupId
                #
            #
            self.__reqObj.setValue("groupid", groupId)
            msgingIo.setGroupId(groupId)
            #
            msgObj = Message.fromReqObj(self.__reqObj, self.__verbose, self.__lfh)
            #
            if self.__verbose:
                logger.info("dep_id is: %s", depId)
            #
            bOk, _bPdbxMdlFlUpdtd, _failedFileRefs = msgingIo.processMsg(msgObj)
            #
            rtrnDict["success"][depId] = "true" if bOk is True else "false"
            #
        bGlobalStatusCheckReqd = False
        #
        # typically only notes actually authored by annotators trigger the Notes indicator in the WFM UI, but annotators have
        # requested that notes generated via emails archived/flagged from BMRB should also trigger the indicator
        #
        if (bOk is True) and (contentType == "notes") and (self._getNotesFlaggingStatus() == "false") and (flagMsg == "y"):
            self.__reqObj.setValue("activate_notes_flagging", "true")
            self._toggleNotesFlagging()
            bGlobalStatusCheckReqd = True
        #
        if (bOk is True) and (contentType == "notes") and (self._getNotesFlaggingStatus("bmrb") == "false") and (flagMsg == "y"):
            self.__reqObj.setValue("activate_notes_flagging_bmrb", "true")
            self._toggleNotesFlagging("bmrb")
            bGlobalStatusCheckReqd = True
        #
        if bGlobalStatusCheckReqd:
            self._checkGlobalMsgStatus()

        rtrnDict["success"]["job"] = "ok"

        rC.addDictionaryItems(rtrnDict)
        #
        return rC

    def _submitMsg(self):
        return self._processMsg("livemsg")

    def _updateDraftState(self):
        return self._processMsg("draft")

    def _processMsg(self, p_msgState):
        """
        :Helpers:

        :Returns:

        """
        #
        #
        self.__getSession()
        #
        rtrnDict = {}
        #
        self.__reqObj.setReturnFormat(return_format="json")
        self.__reqObj.setValue("message_state", p_msgState)  # setting here for downstream processing, used only for processing purposes
        # NOTE: this field is not part of the PdbxMessage data structure, and thus is not persisted to data file.
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgObj = Message.fromReqObj(self.__reqObj, self.__verbose, self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        #
        if self.__verbose:
            logger.info("checking for attached files.")

        for cnt in range(1, 4):
            auxFl = "aux-file" + str(cnt)
            if self.__isFileUpload(auxFl):
                if self.__verbose:
                    logger.info("found attached file.")
                if self.__uploadFile(auxFl):
                    msgObj.fileReferences.append(auxFl)
        #
        if self.__verbose:
            logger.info("msgObj.depositionId is: %s", msgObj.depositionId)
            logger.info("msgObj.messageText is: %r", msgObj.messageText)
            logger.info("msgObj.getFileReferences() is: %r", msgObj.getFileReferences())
        #

        bOk, bPdbxMdlFlUpdtd, failedFileRefs = msgingIo.processMsg(msgObj)
        #
        if bOk:
            rtrnDict["success"] = "true"
            if (msgObj.contentType == "notes") and (self._getNotesFlaggingStatus() == "false"):
                self.__reqObj.setValue("activate_notes_flagging", "true")
                self._toggleNotesFlagging()
        else:
            rtrnDict["success"] = "false"
            #
        rtrnDict["pdbx_model_updated"] = "true" if bPdbxMdlFlUpdtd else "false"
        rtrnDict["append_msg"] = ""
        if (not bOk) and (len(failedFileRefs) > 0):
            sMsg = "Failure to associate message with the following file types: " + (", ".join(failedFileRefs))
            if "aux-file" in failedFileRefs:
                sMsg += "\nIf you are attaching an auxiliary file, please ensure that the file name ends with a known file extension/type."
            rtrnDict["append_msg"] = sMsg

        if bPdbxMdlFlUpdtd:
            wfApi = WfDbApi(verbose=True)
            pw = getdepUIPassword(wfApi, msgObj.depositionId)
            rtrnDict["depid_pw"] = pw

        rC.addDictionaryItems(rtrnDict)
        #
        if self.__verbose:
            logger.info("msgObj.messageId is:%s", msgObj.messageId)
            logger.info("msgObj.depositionId is:%s", msgObj.depositionId)
            logger.info("msgObj.getFileReferences() is: %r", msgObj.getFileReferences())
            logger.info("pdbx_model_updated is: %s", rtrnDict["pdbx_model_updated"])
        #
        return rC

    def _getDepUiPwd(self):
        """
        :Helpers:

        :Returns:

        """
        #
        self.__getSession()
        depId = self.__reqObj.getValue("identifier")
        #
        rtrnDict = {}
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        wfApi = WfDbApi(verbose=True)
        pw = getdepUIPassword(wfApi, depId)
        rtrnDict["depui_pwd"] = pw

        rC.addDictionaryItems(rtrnDict)
        #
        if self.__verbose:
            logger.info("dep_id is:%s", depId)
        #
        return rC

    def __decodeCifToUtf8(self, p_content):
        if sys.version_info[0] < 3:
            h = HTMLParser()  # pylint: disable=used-before-assignment
            return h.unescape(p_content).encode("utf-8")
        else:
            return unescape(p_content.decode("utf-8"))

    def __encodeForSearching(self, p_content):
        ##################################################################
        # we need to accommodate any search filtering taking place
        # at this point we have a resultset of data that was encoded in ascii
        # on its way into cif datafile and then decoded back to UTF8 for
        # return/display in browser.
        # so we must mimic the same process for any search terms before we test
        # such terms for potential matches with the data in the resultset
        ##################################################################
        sSrchAsciiXfrm = p_content.encode("ascii", "xmlcharrefreplace")
        if sSrchAsciiXfrm and len(sSrchAsciiXfrm) > 0:
            if self.__debug:
                logger.debug("sSrchAsciiXfrm is: %r", sSrchAsciiXfrm)

            sSrchUtf8 = self.__decodeCifToUtf8(sSrchAsciiXfrm)
            if self.__debug:
                logger.debug("sSrchUtf8 is: %r", sSrchUtf8)

            return sSrchUtf8
        else:
            return None

    def _markMsgAsRead(self):
        """
        :Helpers:

        :Returns:

        """
        #
        self.__getSession()
        depId = self.__reqObj.getValue("identifier")
        msgId = self.__reqObj.getValue("msg_id")
        actionReqd = self.__reqObj.getValue("action_reqd")
        forReleaseFlg = self.__reqObj.getValue("for_release")

        #
        rtrnDict = {}
        #
        if self.__verbose:
            logger.info(" -- dep_id is:%s", depId)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        actionReqd = "Y" if (actionReqd is None or len(actionReqd) < 1) else actionReqd
        forReleaseFlg = "N" if (forReleaseFlg is None or len(forReleaseFlg) < 1) else forReleaseFlg
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        msgStatusDict = {"message_id": msgId, "deposition_data_set_id": depId, "read_status": "Y", "action_reqd": actionReqd, "for_release": forReleaseFlg}
        bOk = msgingIo.markMsgAsRead(msgStatusDict)
        #
        if bOk:
            rtrnDict["success"] = "true"
        else:
            rtrnDict["success"] = "false"

        rC.addDictionaryItems(rtrnDict)

        return rC

    def _tagMsg(self):
        """
        :Helpers:

        :Returns:

        """
        #
        self.__getSession()
        depId = self.__reqObj.getValue("identifier")
        msgId = self.__reqObj.getValue("msg_id")
        actionReqdFlg = self.__reqObj.getValue("action_reqd")
        readStatusFlg = self.__reqObj.getValue("read_status")
        forReleaseFlg = self.__reqObj.getValue("for_release")
        #
        rtrnDict = {}
        #
        if self.__verbose:
            logger.info(" -- dep_id is:%s", depId)
            logger.info(" -- msgId is:%s", msgId)
            logger.info(" -- actionReqdFlg is:%s", actionReqdFlg)
            logger.info(" -- readStatusFlg is:%s", readStatusFlg)
            logger.info(" -- forReleaseFlg is:%s", forReleaseFlg)
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
        msgStatusDict = {"message_id": msgId, "deposition_data_set_id": depId, "read_status": readStatusFlg, "action_reqd": actionReqdFlg, "for_release": forReleaseFlg}
        bOk = msgingIo.tagMsg(msgStatusDict)
        #
        if bOk:
            rtrnDict["success"] = "true"
        else:
            rtrnDict["success"] = "false"

        rC.addDictionaryItems(rtrnDict)

        return rC

    # Not implemened in backend code
    # def _deleteRowOp(self):
    #     #
    #     rtrnDict = {}
    #     #
    #     if self.__debug:
    #         logger.debug("++++++++++++STARTING at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
    #         #
    #     #
    #     self.__getSession()
    #     depId = self.__reqObj.getValue("identifier")
    #     rowIdx = self.__reqObj.getValue("row_idx")
    #     #
    #     rowIdx = int(rowIdx.replace("row_", ""))
    #     #
    #     if self.__verbose:
    #         logger.info("dep_id is:%s", depId)
    #     #
    #     self.__reqObj.setReturnFormat("json")
    #     rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
    #     #
    #     msgingIo = MessagingIo(self.__reqObj, self.__verbose, self.__lfh)
    #     if self.__debug:
    #         logger.debug("just before call to msgingIo.deleteRow at %s",
    #                      time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
    #         #
    #     ok = msgingIo.deleteRow(depId, rowIdx)
    #     #
    #     if self.__debug:
    #         logger.debug("++++++++++++ just after call to msgingIo.deleteRow at %s",
    #                      time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
    #         #
    #     if ok:
    #         rtrnDict["status"] = "OK"
    #     else:
    #         rtrnDict["status"] = "ERROR"
    #     #
    #     rC.addDictionaryItems(rtrnDict)
    #     #
    #     if self.__debug:
    #         logger.debug("++++++++++++COMPLETING at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
    #         #
    #     return rC

    def _exit_finished(self):
        """Exiting General Annotation Messaging Module when annotator has completed all necessary processing

        CURRENTLY NOT USED

        """
        return self.__exitMessagingMod(mode="completed")

    def _exit_notFinished(self):
        """Exiting General Annotation Messaging Module when annotator has NOT completed all necessary processing
        and user intends to resume use of lig module at another point to continue updating data.

        CURRENTLY NOT USED

        """
        return self.__exitMessagingMod(mode="unfinished")

    ################################################################################################################
    # ------------------------------------------------------------------------------------------------------------
    #      Private helper methods
    # ------------------------------------------------------------------------------------------------------------
    #
    def __uploadFile(self, fileTag="file"):
        #
        #
        if self.__verbose:
            logger.info(" - file upload starting at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

        rawFile = None
        fileName = ""
        #
        # Copy upload file to session directory -
        try:
            rawFile = self.__reqObj.getRawValue(fileTag)
            fileName = str(rawFile.filename)
            #
            # Need to deal with some platform issues -
            #
            if fileName.find("\\") != -1:
                # likely windows path -
                fName = ntpath.basename(fileName)
            else:
                fName = os.path.basename(fileName)

            #
            if self.__verbose:
                logger.info("uploaded file %s", rawFile.filename)
                logger.info("base file   %s", fName)
                #
            # Store upload file in session directory -

            fPathAbs = os.path.join(self.__sessionPath, fName)
            ofh = open(fPathAbs, "wb")
            ofh.write(rawFile.file.read())
            ofh.close()

            for cnt in range(1, 4):
                auxFl = "aux-file" + str(cnt)

                if fileTag == auxFl:
                    fileType = os.path.splitext(fileName)[1].strip(".") if len(os.path.splitext(fileName)[1]) > 1 else "n/a"
                    if fileType.startswith("V") and fileType.split("V")[1].isdigit():
                        fileType = (fileName.rsplit(".V")[0]).rsplit(".")[1]
                        if fileType == "cif":
                            # if file extension is "cif" we handle formally as format type "pdbx"
                            fileType = "pdbx"

                    self.__reqObj.setValue("auxFileName" + str(cnt), fName)
                    self.__reqObj.setValue("auxFilePath" + str(cnt), fPathAbs)
                    self.__reqObj.setValue("auxFileType" + str(cnt), fileType)

                    if self.__verbose:
                        logger.info("auxFileName%s: %s\n", cnt, fileName)
                        logger.info("auxFilePath%s: %s\n", cnt, fPathAbs)
                        logger.info("auxFileType%s: %s\n", cnt, fileType)

        except:  # noqa: E722 pylint: disable=bare-except
            if self.__verbose:
                logger.info("File upload processing failed for %s", str(rawFile.filename))
                logger.exception("In __uploadFile")

            return False

        return True

    def __exitMessagingMod(self, mode):
        """CURRENTLY NOT USED

        Function to accommodate user request to exit messaging module,
        close interface, and return to workflow manager interface.
        Supports different 'modes' = ('completed' | 'unfinished')

        :Params:
            ``mode``:
                'completed' if annotator has completed all edits to mmCif data and wishes to
                    conclude work in the General Annotation mmCif Messaging.
                'unfinished' if annotator wishes to leave General Annotation mmCif Messaging but resume work at a later point.

        :Returns:
            ResponseContent object.
        """
        if self.__verbose:
            logger.info("--------------------------------------------")
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - starting")
        #
        if mode == "completed":
            state = "closed(0)"
        elif mode == "unfinished":
            state = "waiting"
        #
        bIsWorkflow = self.__isWorkflow()
        #
        self.__getSession()
        sessionId = self.__sessionId
        depId = self.__reqObj.getValue("identifier")
        instId = self.__reqObj.getValue("instance")
        classId = self.__reqObj.getValue("classID")
        fileSource = str(self.__reqObj.getValue("filesource")).strip().lower()
        #
        if self.__verbose:
            logger.info("--------------------------------------------\n")
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - depId   %s", depId)
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - instId  %s", instId)
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - classID %s", classId)
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - sessionID %s", sessionId)
            logger.info("+MessagingWebAppWorker.__exitMessagingMod() - filesource %r", fileSource)

        #
        self.__reqObj.setReturnFormat("json")
        #
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        # Update WF status database and persist chem comp assignment states -- ONLY if lig module was running in context of wf-engine
        #
        if bIsWorkflow:
            try:
                bOkay = self.__saveMessagingModState()
                if bOkay:
                    bSuccess = self.__updateWfTrackingDb(state)
                    if not bSuccess:
                        rC.setError(errMsg="+MessagingWebAppWorker.__exitMessagingMod() - TRACKING status, update to '%s' failed for session %s \n" % (state, sessionId))
                else:
                    rC.setError(errMsg="+MessagingWebAppWorker.__exitMessagingMod() - problem saving cif file")

            except:  # noqa: E722 pylint: disable=bare-except
                if self.__verbose:
                    logger.info("+MessagingWebAppWorker.__exitMessagingMod() - problem saving cif file")
                traceback.print_exc(file=self.__lfh)
                rC.setError(errMsg="+MessagingWebAppWorker.__exitMessagingMod() - exception thrown on saving cif file")

        else:
            try:
                bOkay = self.__saveMessagingModState()
                if bOkay:
                    if self.__verbose:
                        logger.info("successfully saved cif file to session directory %s at %s", self.__sessionPath, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
                else:
                    if self.__verbose:
                        logger.info("failed to save cif file to session directory %s at %s", self.__sessionPath, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
                    rC.setError(errMsg="+MessagingWebAppWorker.__exitMessagingMod() - problem saving cif file")

            except:  # noqa: E722 pylint: disable=bare-except
                if self.__verbose:
                    logger.info("failed to save cif file to session directory %s at %s", self.__sessionPath, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
                logger.exception("Failed to save cif file to session directory %s at %s", self.__sessionPath, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
                rC.setError(errMsg="+MessagingWebAppWorker.__exitMessagingMod() - exception thrown on saving cif file")
            if self.__verbose:
                logger.info("+MessagingWebAppWorker.__exitMessagingMod() - Not in WF environ so skipping status update to TRACKING database for session %s", sessionId)
        #
        return rC

    def __updateWfTrackingDb(self, p_status):  # pylint: disable=unused-argument
        """Private function used to udpate the Workflow Status Tracking Database

        :Params:
            ``p_status``: the new status value to which the deposition data set is being set

        :Helpers:
            wwpdb.apps.msgmodule.utils.WfTracking.WfTracking

        :Returns:
            ``bSuccess``: boolean indicating success/failure of the database update
        """
        #
        bSuccess = False
        #
        # sessionId = self.__sessionId
        # depId = self.__reqObj.getValue("identifier").upper()
        # instId = self.__reqObj.getValue("instance")
        # classId = str(self.__reqObj.getValue("classID")).lower()
        #
        # try:
        #     wft = WfTracking(verbose=self.__verbose, log=self.__lfh)
        #     wft.setInstanceStatus(depId=depId, instId=instId, classId=classId, status=p_status)
        #     bSuccess = True
        #     if self.__verbose:
        #         self.__lfh.write("+MessagingWebAppWorker.__updateWfTrackingDb() -TRACKING status updated to '%s' for session %s \n" % (p_status, sessionId))
        # except:  # noqa: E722 pylint: disable=bare-except
        #     bSuccess = False
        #     if self.__verbose:
        #         self.__lfh.write("+MessagingWebAppWorker.__updateWfTrackingDb() - TRACKING status, update to '%s' failed for session %s \n" % (p_status, sessionId))
        #     traceback.print_exc(file=self.__lfh)
        #     #
        return bSuccess

    def __saveMessagingModState(self):
        """PLACEHOLDER"""
        # """
        # exprtDirPath=None
        # exprtFilePath=None
        # fileSource = str(self.__reqObj.getValue("filesource")).strip().lower()
        # depId  =  self.__reqObj.getValue("identifier")
        # instId =  self.__reqObj.getValue("instance")
        # #classId = self.__reqObj.getValue("classid")
        # sessionId=self.__sessionId
        # #
        # if (self.__verbose):
        #     self.__lfh.write("--------------------------------------------\n")
        #     self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() - dataFile   %s \n" % dataFile)
        # #
        # if fileSource in ['archive','wf-archive','wf_archive']:
        #     self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() - processing archive | filesource %r \n" % fileSource)
        #     dfRef=DataFileReference()
        #     dfRef.setDepositionDataSetId(depId)
        #     dfRef.setStorageType('archive')
        #     dfRef.setContentTypeAndFormat('model','pdbx')
        #     dfRef.setVersionId('latest')

        #     if (dfRef.isReferenceValid()):
        #         exprtDirPath=dfRef.getDirPathReference()
        #         exprtFilePath=dfRef.getFilePathReference()
        #         sP=dfRef.getSitePrefix()
        #         if (self.__verbose):
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() site prefix             : %s\n" % sP)
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() CC assign details export directory path: %s\n" % exprtDirPath)
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() CC assign details export file      path: %s\n" % exprtFilePath)
        #     else:
        #         self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() Bad archival reference for id %s \n" % depId)

        # elif (fileSource in ['wf-instance','wf_instance']):
        #     self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() - processing instance | filesource %r \n" % fileSource)
        #     dfRef=DataFileReference()
        #     dfRef.setDepositionDataSetId(depId)
        #     dfRef.setWorkflowInstanceId(instId)
        #     dfRef.setStorageType('wf-instance')
        #     dfRef.setContentTypeAndFormat('model','pdbx')
        #     dfRef.setVersionId('latest')

        #     if (dfRef.isReferenceValid()):
        #         exprtDirPath=dfRef.getDirPathReference()
        #         exprtFilePath=dfRef.getFilePathReference()
        #         sP=dfRef.getSitePrefix()
        #         if (self.__verbose):
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() site prefix             : %s\n" % sP)
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() CC assign details export directory path: %s\n" % exprtDirPath)
        #             self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() CC assign details export           path: %s\n" % exprtFilePath)
        #     else:
        #         self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() Bad wf-instance reference for id %s wf id %s\n" % (depId,instId))
        # elif (fileSource in ['rcsb_dev']):
        #     self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() - processing for 'rcsb_dev' filesource.\n")
        #     exprtDirPath=self.__sessionPath
        #     exprtFilePath=os.path.join(self.__sessionPath,"rcsb_dev_testCifFileSave.cif")
        # else:
        #     self.__lfh.write("+MessagingWebAppWorker.__saveMessagingModState() - processing undefined | filesource %r \n" % fileSource)
        #     exprtDirPath=self.__sessionPath
        #     exprtFilePath=os.path.join(self.__sessionPath,dataFile)

        # # export updated mmCif file here
        # # bOk = callExportFile Function
        # msgingIo = MessagingIo(self.__reqObj,self.__verbose,self.__lfh)
        # bOk = msgingIo.doExport(exprtDirPath,exprtFilePath)
        # """
        return True

    def __isFileUpload(self, fileTag="file"):
        """Generic check for the existence of request paramenter "file"."""
        try:
            stringtypes = (unicode, str)
        except NameError:
            stringtypes = (str, bytes)

        # Gracefully exit if no file is provide in the request object -
        fs = self.__reqObj.getRawValue(fileTag)
        if self.__verbose and self.__debug:
            logger.debug("+MessagingWebApp.__isFileUpload() - type of 'fs' is %s", type(fs))

        if (fs is None) or isinstance(fs, stringtypes):
            return False
        if self.__verbose and self.__debug:
            logger.debug("+MessagingWebApp.__isFileUpload() - file upload is found to be True")
        return True

    def __getSession(self):
        """Join existing session or create new session as required."""
        #
        self.__sObj = self.__reqObj.newSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        # self.__rltvSessionPath = self.__sObj.getRelativePath()
        if self.__verbose:
            logger.info("------------------------------------------------------")
            logger.info("+MessagingWebApp.__getSession() - creating/joining session %s", self.__sessionId)
            # logger.info("+MessagingWebApp.__getSession() - workflow storage path    %s\n" % self.__workflowStoragePath)
            logger.info("+MessagingWebApp.__getSession() - session path %s", self.__sessionPath)

    # def __setSemaphore(self):
    #     sVal = str(time.strftime("TMP_%Y%m%d%H%M%S", time.localtime()))
    #     self.__reqObj.setValue("semaphore", sVal)
    #     return sVal

    # def __openSemaphoreLog(self, semaphore="TMP_"):
    #     # sessionId = self.__reqObj.getSessionId()
    #     fPathAbs = os.path.join(self.__sessionPath, semaphore + ".log")
    #     self.__lfh = open(fPathAbs, "w")

    # def __closeSemaphoreLog(self, semaphore="TMP_"):
    #     self.__lfh.flush()
    #     self.__lfh.close()

    # def __postSemaphore(self, semaphore="TMP_", value="OK"):
    #     # sessionId = self.__reqObj.getSessionId()
    #     fPathAbs = os.path.join(self.__sessionPath, semaphore)
    #     fp = open(fPathAbs, "w")
    #     fp.write("%s\n" % value)
    #     fp.close()
    #     return semaphore

    # def __semaphoreExists(self, semaphore="TMP_"):
    #     # sessionId = self.__reqObj.getSessionId()
    #     fPathAbs = os.path.join(self.__sessionPath, semaphore)
    #     if os.access(fPathAbs, os.F_OK):
    #         return True
    #     else:
    #         return False

    # def __getSemaphore(self, semaphore="TMP_"):
    #     # sessionId = self.__reqObj.getSessionId()
    #     fPathAbs = os.path.join(self.__sessionPath, semaphore)
    #     if self.__verbose:
    #         self.__lfh.write("+MessagingWebApp.__getSemaphore() - checking %s in path %s\n" % (semaphore, fPathAbs))
    #     try:
    #         fp = open(fPathAbs, "r")
    #         lines = fp.readlines()
    #         fp.close()
    #         sval = lines[0][:-1]
    #     except:  # noqa: E722 pylint: disable=bare-except
    #         sval = "FAIL"
    #     return sval

    # def __openChildProcessLog(self, label="TMP_"):
    #     # sessionId = self.__reqObj.getSessionId()
    #     fPathAbs = os.path.join(self.__sessionPath, label + ".log")
    #     self.__lfh = open(fPathAbs, "w")

    # def __processTemplate(self, fn, parameterDict={}):
    #     """Read the input HTML template data file and perform the key/value substitutions in the
    #     input parameter dictionary.

    #     :Params:
    #         ``parameterDict``: dictionary where
    #         key = name of subsitution placeholder in the template and
    #         value = data to be used to substitute information for the placeholder

    #     :Returns:
    #         string representing entirety of content with subsitution placeholders now replaced with data
    #     """
    #     tPath = self.__reqObj.getValue("TemplatePath")
    #     fPath = os.path.join(tPath, fn)
    #     ifh = open(fPath, "r")
    #     sIn = ifh.read()
    #     ifh.close()
    #     return sIn % parameterDict

    def __isWorkflow(self):
        """Determine if currently operating in Workflow Managed environment

        :Returns:
            boolean indicating whether or not currently operating in Workflow Managed environment
        """
        #
        fileSource = str(self.__reqObj.getValue("filesource")).lower()
        #
        if self.__verbose:
            logger.info("+MessagingWebAppWorker.__isWorkflow() - filesource is %s", fileSource)
        #
        # add wf_archive to fix PDBe wfm issue -- jdw 2011-06-30
        #
        if fileSource in ["archive", "wf-archive", "wf_archive", "wf-instance", "wf_instance"]:
            # if the file source is any of the above then we are in the workflow manager environment
            return True
        else:
            # else we are in the standalone dev environment
            return False


class RedirectDevice:
    def write(self, s):
        pass


def main_test():
    sTool = MessagingWebApp()
    d = sTool.doOp()
    for k, v in d.items():
        sys.stdout.write("Key - %s  value - %r\n" % (k, v))


if __name__ == "__main__":
    main_test()



#
# File:     doServiceRequestWebOb.wsgi
# Created:  26-Sep-2018
#
# Updated:
# 26-Sep-2018 EP    Ported from fcgi version
"""
This top-level responder for requests to /services/.... url for the
wwPDB General Annotation editor application framework.
This version depends on WSGI and WebOb.
Adapted from mod_wsgi version -
"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys
import traceback
import logging

from webob import Request, Response
from wwpdb.apps.msgmodule.webapp.MessagingWebApp import MessagingWebApp
from wwpdb.utils.config.ConfigInfo import getSiteId

# Create logger
FORMAT = "[%(levelname)s]-%(module)s.%(funcName)s: %(message)s"
logging.basicConfig(format=FORMAT)
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

#  - URL mapping and application specific classes are launched from MessagingWebApp()

l2 = logging.getLogger("wwpdb.apps.msgmodule.io.MessagingDataImport")
l2.setLevel(logging.INFO)
l2 = logging.getLogger("wwpdb.apps.msgmodule.io.MessagingDataExport")
l2.setLevel(logging.INFO)


class MyRequestApp(object):
    """Handle server interaction using FCGI/WSGI and WebOb Request
    and Response objects.
    """

    def __init__(self, textString="doServiceRequest() - WebOb version", verbose=True, log=sys.stderr):
        """ """
        self.__text = textString
        self.__verbose = verbose
        self.__lfh = log

    def __dumpEnv(self, request):
        outL = []
        # outL.append('<pre align="left">')
        outL.append("\n------------------doServiceRequest()------------------------------\n")
        outL.append("Web server request data content:\n")
        outL.append("Text initialization:   %s\n" % self.__text)
        try:
            outL.append("Host:         %s\n" % request.host)
            outL.append("Path:         %s\n" % request.path)
            outL.append("Method:       %s\n" % request.method)
            outL.append("Query string: %s\n" % request.query_string)
            outL.append("Parameter List:\n")
            for name, value in request.params.items():
                outL.append("Request parameter:    %s:  %r\n" % (name, value))
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)

        outL.append("\n------------------------------------------------\n\n")
        # outL.append("</pre>")
        return outL

    def __call__(self, environment, responseApplication):
        """WSGI callable entry point"""
        myRequest = Request(environment)
        #
        myParameterDict = {}
        siteId = getSiteId()
        try:
            if "WWPDB_SITE_ID" in environment:
                siteId = environment["WWPDB_SITE_ID"]
                self.__lfh.write("+MyRequestApp.__call__() - WWPDB_SITE_ID environ variable captured as %s\n" % siteId)
            # """
            # for name,value in environment.items():
            #     self.__lfh.write("+MyRequestApp.__call__() - ENVIRON parameter:    %s:  %r\n" % (name,value))
            # """
            for name, value in myRequest.params.items():
                if name not in myParameterDict:
                    myParameterDict[name] = []
                myParameterDict[name].append(value)
                self.__lfh.write("+MyRequestApp.__call__() - REQUEST parameter:    %s:  %r\n" % (name, value))
            myParameterDict["request_path"] = [myRequest.path.lower()]
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)
            self.__lfh.write("+MyRequestApp.__call__() - contents of request data\n")
            self.__lfh.write("%s" % ("".join(self.__dumpEnv(request=myRequest))))

        ###
        #  At this point we have everything needed from the request !
        ###
        myResponse = Response()
        myResponse.status = "200 OK"
        myResponse.content_type = "text/html"
        ###
        #   Application specific functionality called here --
        #   Application receives path and parameter info only!
        ###
        msgmodule = MessagingWebApp(parameterDict=myParameterDict, verbose=self.__verbose, log=self.__lfh, siteId=siteId)
        rspD = msgmodule.doOp()
        myResponse.content_type = rspD["CONTENT_TYPE"]

        if sys.version_info[0] > 2:
            if isinstance(rspD["RETURN_STRING"], str):
                myResponse.text = rspD["RETURN_STRING"]
            else:
                myResponse.body = rspD["RETURN_STRING"]
        else:
            myResponse.body = rspD["RETURN_STRING"]

        ####
        ###
        return myResponse(environment, responseApplication)


##
#   NOTE -  verbose setting is set here ONLY!
##
application = MyRequestApp(textString="doServiceRequest() - WebOb version", verbose=True, log=sys.stderr)



##
# File:  EmHeaderUtils.py
# Date:  20-Oct-2015  J. Westbrook
#
# Update:
#       21-Feb-2016  jdw add releaseHeaderPdbx()
#       23-May-2018  ep  Update to V3 translator API and use logging
"""
Manage map header file operations -


"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys

# import os.path
import traceback
import logging

# import shutil
# from wwpdb.utils.config.ConfigInfo import ConfigInfo
# from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppEm
# from mmcif_utils.trans.InstanceMapper import InstanceMapper
from wwpdb.utils.emdb.cif_emdb_translator.cif_emdb_translator import CifEMDBTranslator

#

logger = logging.getLogger(__name__)


class EmHeaderUtils(object):
    """
    Manage map header file operations -

    """

    def __init__(self, siteId, verbose=False, log=sys.stderr):
        pass
        # self.__lfh = log
        # self.__siteId = siteId
        # self.__cI = ConfigInfo(self.__siteId)
        # self.__cIA = ConfigInfoAppEm(self.__siteId)
        # self.__mappingFilePath = self.__cIA.get_emd_mapping_file_path()

    # def __mkdir(self, path):
    #     if not os.path.isdir(path):
    #         os.makedirs(path, 0o755)

    # def releaseHeader(self, filePath, emdbId):
    #     ok = False
    #     try:
    #         logger.info("starting for %s", filePath)
    #         upId = str(emdbId).upper()
    #         pth = os.path.join(self.__cI.get("SITE_ARCHIVE_STORAGE_PATH"), "for_release", "emd", upId, "header")
    #         self.__mkdir(pth)
    #         lcId = str(emdbId).lower()
    #         mod_lcId = lcId.replace("-", "_")
    #         fp = os.path.join(pth, mod_lcId + "_v3.xml")
    #         logger.info("copied %s to %s", filePath, fp)
    #         shutil.copyfile(filePath, fp)
    #         ok = True
    #     except:  # noqa: E722
    #         logger.exception("failing for %s", filePath)

    #     return ok

    # def releaseHeaderPdbx(self, filePath, emdbId):
    #     ok = False
    #     try:
    #         logger.info("starting for %s", filePath)
    #         upId = str(emdbId).upper()
    #         pth = os.path.join(self.__cI.get("SITE_ARCHIVE_STORAGE_PATH"), "for_release", "emd", upId)
    #         self.__mkdir(pth)
    #         lcId = str(emdbId).lower()
    #         fp = os.path.join(pth, lcId + ".cif")
    #         logger.info("copied %s to %s", filePath, fp)
    #         shutil.copyfile(filePath, fp)
    #         ok = True
    #     except:  # noqa: E722
    #         logger.exception("failing for %s", filePath)

    #     return ok

    # def transEmd(self, inpFilePath, outFilePath, mode="src-dst", tags=None):
    #     if tags is None:
    #         tags = []

    #     ok = False
    #     try:
    #         im = InstanceMapper(verbose=True, log=sys.stderr)
    #         im.setMappingFilePath(self.__mappingFilePath)
    #         if tags != []:
    #             im.setFilterTagList(tags)
    #         ok = im.translate(inpFilePath, outFilePath, mode=mode)
    #     except:  # noqa: E722
    #         logger.exception("failing for %s", inpFilePath)

    #     return ok

    def transHeader(self, inpFilePath, outFilePath, logFilePath, validateXml=True):
        ok = False
        logging.info("Starting conversion for %s", inpFilePath)
        #
        try:
            translator = CifEMDBTranslator()
            translator.set_logger_logging(log_error=True, error_log_file_name=logFilePath)
            translator.read_emd_map_v2_cif_file()
            if validateXml:
                translator.translate_and_validate(in_cif=inpFilePath, out_xml=outFilePath)
            else:
                translator.translate(in_cif=inpFilePath, out_xml=outFilePath)
            # This will close the output file
            translator.write_logger_logs(write_error_log=True)
            if translator.is_translation_log_empty:
                ok = True
            else:
                ok = False
        except:  # noqa: E722  pylint: disable=bare-except
            logger.exception("Map header translation failed for %s", inpFilePath)
            se = traceback.format_exc()
            fOut = open(logFilePath, "w")
            fOut.write("Exception during XML production\n")
            fOut.write(se)
            fOut.close()

        return ok



##
# File:  MessagingTemplates.py
# Date:  12-Aug-2014
# Updates:
#
#    2014-10-10    RPS    msgTmplt_correspondence abbreviated to eliminate boilerplate that is already contained in correspondence content.
#    2014-10-30    RPS    Removed square brackets surrounding PDB and Deposition IDs
#    2014-11-18    RPS    Now using single template for validation letter and initializing in this class. (i.e. did away with validation-minor vs major issues templates).
#                            New msgTmplt_systemUnlocked template.
#    2015-01-28    RPS    Updated language for approval templates to remove line: "Please acknowledge receipt of this message."
#    2015-03-11    RPS    Improving display of entry title within horiz rules in message templates
#    2015-05-08    RPS    Introducing support for custom message templates relative to different experimental methods.
#    2015-06-12    RPS    "System Unlocked" template updated.
#    2015-09-17    RPS    Migrating template pieces used for email notifications from MessagingIo to this module.
#    2015-10-27    RPS    Introduction of EM specific message templates.
#    2015-10-28    RPS    More updates to support EM specific message templates.
#    2015-12-02    RPS    Removed obsolete validation letter content (validation content no longer generated by Msgmodule)
#    2015-12-10    RPS    Updated manner in which obtaining URL for deposition webpage
#    2016-01-24    RPS    Updated for new template for EM map-only, post-annotation letter
#    2016-03-23    EP     Update urls for policy and procedure links
#    2016-04-21    RPS    Adjusting msgTmplt_systemUnlocked_em to remove clause referring to validation server instructions.
#    2016-05-10    RPS    release message tmplts updated to indicate that release cannot happen sooner than stated dates.
#    2016-08-09    RPS    Changes to support site/annotator specific footers in message templates.
#                            Updates to EM letter templates in order to correctly distinguish between map-only and map+model releases.
#    2016-09-21    RPS    Integrating customized signoff with msgTmplt_vldtn template.
#    2017-08-18    RPS    Accommodating updates in "withdrawn" letter template
#    2017-10-09    RPS    Updating template for "explicit approval". Adjusting signoff content for EM Map Only cases.
#    2018-01-30    RPS    Readjusting signoff content for EM Map Only cases.
#    2019-06-19    EP     Add msgTmplt_remindUnlocked for automatic sending of message for unlocked
#    2022-01-26    CS     Update template
#    2022-02-27    CS     Update template for EM only withdrawn
##
"""
Convenience class to serve as source of message templates for dynamic population

"""
__docformat__ = "restructuredtext en"
__author__ = "Raul Sala"
__email__ = "rsala@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.02"


class MessagingTemplates(object):
    """Convenience class to serve as source of message templates for dynamic population"""

    msgTmplt_closing = """Sincerely yours,

%(full_name_annotator)s

%(annotator_group_signoff)s
%(site_contact_details)s
"""
    msgTmplt_annotatorGroupSignoff = "The wwPDB Biocuration Staff"

    msgTmplt_annotatorGuestPdbjSignoff = "Guest Biocurators of PDBj"

    msgTmplt_site_contact_details_rcsb = """
--------------------------------------------------------------------------------------------------------------
RCSB Protein Data Bank (RCSB PDB), a wwPDB partner site
http://www.rcsb.org
Rutgers, The State University of New Jersey, Piscataway, NJ, USA
Facebook: http://www.facebook.com/RCSBPDB
Twitter: http://twitter.com/#!/buildmodels
--------------------------------------------------------------------------------------------------------------"""

    msgTmplt_site_contact_details_pdbe = """
--------------------------------------------------------------------------------------------------------------
Protein Data Bank in Europe (PDBe), a wwPDB partner site
http://www.PDBe.org

EMBL Outstation Hinxton
European Bioinformatics Institute
Wellcome Trust Genome Campus
Hinxton
Cambridge CB10 1SD UK

http://www.facebook.com/proteindatabank
http://twitter.com/PDBeurope
--------------------------------------------------------------------------------------------------------------"""

    msgTmplt_site_contact_details_pdbj = """
--------------------------------------------------------------------------------------------------------------
Protein Data Bank Japan (PDBj), a wwPDB partner site
http://pdbj.org
Institute for Protein Research, Osaka University, Osaka, Japan
Facebook: https://www.facebook.com/PDBjapan
Twitter: https://twitter.com/PDBj_en
--------------------------------------------------------------------------------------------------------------"""

    msgTmplt_default = """Dear Depositors,

Please acknowledge receipt of this message.


This message concerns your structure %(accession_ids)s (Deposition ID %(identifier)s) entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

Entry authors:

%(entry_authors_newline_list)s


Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_vldtn = """%(starter_msg_body)s

%(msg_closing)s
"""

    msgTmplt_approvalExplicit = """Dear Depositors,

Thank you for your response, this message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) has been approved. The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

We have changed the status of the entry to %(status_code)s (%(entry_status)s). Your structure is now on hold until %(expire_date)s. It may be released earlier if you notify us that the structure is published or can be released.

%(doinotice)s
Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_approvalImplicit = """Dear Depositors,

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) has been approved. The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

As we have not heard from you since we sent the validation report and processed files to you on %(outbound_rprt_date)s, we have changed the status of the entry to %(status_code)s (%(entry_status)s) with implicit approval in accordance with wwPDB policies, http://www.wwpdb.org/documentation/policy.

Your structure is now on hold until %(expire_date)s. It may be released earlier if you notify us that the structure is published or can be released.

%(doinotice)s
Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_reminder = """Dear Depositors,

Please acknowledge receipt of this message.

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) is still awaiting your input. The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

We have not heard from you since we sent you the validation report, processed PDB and mmCIF files on %(outbound_rprt_date)s. Please respond to the questions raised in our previous correspondence as soon as possible.

In accordance with wwPDB policies (http://www.wwpdb.org/documentation/policy), if we find a publication describing the entry, it will be released with CAVEAT records highlighting any outstanding issues.  If after %(expire_date)s (one year from the deposition date) we are unable to find a publication describing this structure, the entry may be withdrawn if there remain outstanding issues, otherwise the entry will be released.

Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_releaseWthPblctn = """Dear Depositors,

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code)s', will be released on %(release_date)s. This is the next available release date.

The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

Your entry has been updated with the following primary citation:

Author(s):   %(citation_authors)s

Title:   %(citation_title)s

Journal:   %(citation_journal_abbrev)s%(citation_journal_volume)s%(citation_pages)s%(citation_year)s%(citation_pubmedid)s%(citation_doi)s

%(doinotice)s%(thurs_prerelease_clause)s

%(msg_closing)s
"""

    msgTmplt_releaseWthOutPblctn = """Dear Depositors,

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited on %(recvd_date)s with release instructions, '%(auth_rel_status_code)s', will be released on %(release_date)s.  This is the next available release date.

The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

%(doinotice)s%(thurs_prerelease_clause)s

%(msg_closing)s
"""

    msgTmplt_withdrawn = """Dear Depositors,

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code)s', will be withdrawn on %(withdrawn_date)s.

The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

Please do not use these accession IDs in any publications.
%(thurs_wdrn_clause)s

%(msg_closing)s
"""

    msgTmplt_systemUnlocked = """Dear Depositor,

The deposition interface has been unlocked as per your request. If you are providing replacement coordinates, please visit the validation server (http://wwpdb-validation.wwpdb.org/validservice/)
and check your coordinates to ensure the quality of your structure prior to re-submission.  Make sure to press the 'Submit deposition' button once you have finished making your changes within the deposition interface.

%(msg_closing)s
"""

    msgTmplt_systemUnlockedPostRel = """Dear Depositor,

The deposition interface has been unlocked as per your request in order to facilitate replacement of your model. You will be able to upload a replacement model, without any change to experimental data. As this entry has already been released, we will treat this as an entry that is to be released as soon as revisions are complete.
Make sure to press the 'Submit deposition' button once you have finished making your changes within the deposition interface.

%(msg_closing)s
"""

    msgTmplt_remindUnlocked = """Dear Depositors,

Please acknowledge receipt of this message.

This message is to inform you that your structure %(accession_ids)s (Deposition ID %(identifier)s) is still awaiting your input. The entry is entitled:

%(horiz_line)s
%(title)s
%(horiz_line)s

This entry was unlocked on %(unlock_date)s per your request. However, the updated files and/or corrections have not been submitted yet. Please upload the new files, make corrections to the data in the deposition interface if needed, and press the 'Submit deposition' button so that we can finalize the processing of your deposition.

Thank you for your attention.

%(msg_closing)s
"""

    ######################################################################
    # TEMPLATE COMPONENTS BELOW ARE USED FOR ELECTRON MICROSCOPY ENTRIES #
    ######################################################################

    msgTmplt_closing_emMapOnly = """Sincerely yours,

%(full_name_annotator)s

%(annotator_group_signoff)s
%(site_contact_details)s
"""

    msgTmplt_annotatorGroupSignoff_em = "The wwPDB Biocuration Staff"

    msgTmplt_annotatorGuestPdbjSignoff_em = "Guest Biocurators of PDBj"

    msgTmplt_default_em = """Dear Depositors,

Please acknowledge receipt of this message.

This message concerns your %(accession_ids)s (Deposition ID %(identifier)s) entitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

Entry authors:

%(em_entry_authors_newline_list)s


Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_mapOnly_authStatus_em = """Dear Depositors,

Your %(accession_ids)s which corresponds to Deposition ID %(identifier)s has been annotated and the status is now AUTH.

%(horiz_line)s
%(em_title)s
%(horiz_line)s

Entry authors:

%(em_entry_authors_newline_list)s

Please review the curated data and either reply with your approval or let us know if any additional changes are required. Your approval will be assumed if we do not hear from you within three weeks from the time when the annotation report is made available and assuming there are no major issues with the submission.

When the primary citation associated with your entry is published, please notify us through the deposition system and provide the PubMed ID (if available), journal name, volume, page numbers, title, authors list and DOI.

Please be aware that all public access to your entry will be via the accession code, %(accession_ids)s, which is distinct from the deposition number.

Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_approvalExplicit_em = """Dear Depositors,

Thank you for your response, this message is to inform you that your %(accession_ids)s (Deposition ID %(identifier)s) %(has_have)sbeen approved. The%(entry_entries)s%(is_are)sentitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

We have changed the status of the entry to %(status_code_em_map)s (%(entry_status_em_map)s). Your%(entry_entries)s%(is_are)snow on hold until %(expire_date_em_map)s.%(it_they)smay be released earlier if you notify us that the entry is published or can be released.

%(doinotice)s
Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_approvalImplicit_em = """Dear Depositors,

This message is to inform you that your %(accession_ids)s (Deposition ID %(identifier)s) %(has_have)sbeen approved. The%(entry_entries)s%(is_are)sentitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

As we have not heard from you since we sent the %(vldtn_rprt)sprocessed files to you on %(outbound_rprt_date_em)s, we have changed the status of the entry to %(status_code_em_map)s (%(entry_status_em_map)s) with implicit approval in accordance with wwPDB policies, http://www.wwpdb.org/documentation/policy.html.

Your%(entry_entries)s%(is_are)snow on hold until %(expire_date_em_map)s.%(it_they)smay be released earlier if you notify us that the entry is published or can be released.

%(doinotice)s
Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_reminder_em = """Dear Depositors,

Please acknowledge receipt of this message.

This message is to inform you that your %(accession_ids)s (Deposition ID %(identifier)s) %(is_are)sstill awaiting your input. The entry is entitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

We have not heard from you since we sent you the %(vldtn_rprt)sprocessed files on %(outbound_rprt_date_em)s. Please respond to the questions raised in our previous correspondence as soon as possible.

In accordance with wwPDB policies (http://www.wwpdb.org/documentation/policy.html), if we find a publication describing the %(entry_entries_comma)s%(it_they_lcase)swill be released%(caveat_records)s.
If after %(expire_date_em_map)s (one year from the deposition date) we are unable to find a publication describing%(this_these)s%(entry_entries_comma)s the%(entry_entries)smay be withdrawn if there remain outstanding issues, otherwise the%(entry_entries)swill be released.

Thank you for your attention.

%(msg_closing)s
"""

    msgTmplt_releaseWthPblctn_em = """Dear Depositors,

This message is to inform you that your %(accession_ids_em_rel)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code_em_rel)s', will be released on %(release_date)s.  This is the next available release date.

The%(entry_entries_em_rel)s%(is_are_em_rel)sentitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

Your%(entry_entries_em_rel)s%(has_have_em_rel)sbeen updated with the following primary citation:

Author(s):   %(citation_authors)s

Title:   %(citation_title)s

Journal:   %(citation_journal_abbrev)s%(citation_journal_volume)s%(citation_pages)s%(citation_year)s%(citation_pubmedid)s%(citation_doi)s

%(doinotice)s%(thurs_prerelease_clause)s

%(msg_closing)s
"""

    msgTmplt_releaseWthOutPblctn_em = """Dear Depositors,

This message is to inform you that your %(accession_ids_em_rel)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code_em_rel)s', will be released on %(release_date)s.  This is the next available release date.

The%(entry_entries_em_rel)s%(is_are_em_rel)sentitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

%(doinotice)s%(thurs_prerelease_clause)s

%(msg_closing)s
"""

    msgTmplt_withdrawn_em = """Dear Depositors,

This message is to inform you that your %(accession_ids)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code)s', will be withdrawn on %(withdrawn_date)s.

The entry is entitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

Please do not use these accession IDs in any publications.

%(thurs_wdrn_clause)s

%(msg_closing)s
"""

    msgTmplt_withdrawn_em_map_only = """Dear Depositors,

This message is to inform you that your %(accession_ids)s (Deposition ID %(identifier)s) and the associated experimental data which were deposited with release instructions, '%(auth_rel_status_code)s', will be withdrawn on %(withdrawn_date)s.

The entry is entitled:

%(horiz_line)s
%(em_title)s
%(horiz_line)s

Please do not use these accession IDs in any publications.

%(thurs_wdrn_clause_em_map_only)s

%(msg_closing)s
"""

    msgTmplt_systemUnlocked_em = """Dear Depositor,

The deposition interface has been unlocked as per your request. Make sure to press the 'Submit deposition' button once you have finished making your changes within the deposition interface.

%(msg_closing)s
"""

    ####################################################################################################################
    # TEMPLATE COMPONENTS BELOW ARE USED FOR EMAIL NOTIFICATIONS THAT ACCOMPANY MESSAGES FROM ANNOTATORS TO DEPOSITORS #
    ####################################################################################################################

    emailNotif_msgTmplt = """From: <%(sender)s>
To: %(receiver)s
Subject: %(subject)s
%(mime_hdr)s%(msg_content)s
"""

    emailNotif_mimeHdr = "MIME-Version: 1.0\n"
    emailNotif_msgBodyMimeSpec = """Content-Type: text/html; charset="UTF-8"
Content-Transfer-Encoding:8bit
"""
    emailNotif_replyRedirectFooter = """<br /><hr /><br />
In order to reply to the above message you must log into the wwPDB Deposition System at:<br /><br />

   %(tab)s%(dep_email_url)s<br /><br />

"""
    emailNotif_msgBodyMain = (
        """Dear %(email_to_lname)s,<br /><br />

Please be advised that you have a notification message requiring your attention<br />
as relates to your wwPDB deposition with dataset ID: %(dep_id)s%(pdb_block)s
%(entry_title_block)s
%(entry_authors_block)s
The subject of the message is: "%(comm_subject)s"<br /><br />

In order to view the given message %(mention_vldtn_rprts)s in detail, and send any replies, please log into the wwPDB Deposition System at:<br /><br />

   %(tab)s%(dep_email_url)s<br /><br />

and then use the left-hand navigation panel to visit the "Communication" section.<br /><br />

The body of the message is provided below for your convenience:<br /><br /><hr />

%(orig_commui_msg_content)s"""
        + emailNotif_replyRedirectFooter
    )

    emailNotif_msgBodyTmplt = """%(msg_mime_spec)s

%(msg_body_main)s

"""
    #
    emailNotif_pdbBlock = """ and PDB ID: %s<br /><br />

"""
    emailNotif_entryTitleBlock = """Entry Title:<br /><br />

       %(tab)s%(entry_title)s<br /><br />

"""
    emailNotif_entryAuthorsBlock = """Entry Authors:<br /><br />

       %(tab)s%(entry_authors)s<br /><br />

"""



##
#
# File:    DateUtil.py
# Author:  E. Peisach
# Date:    17-Nov-2018
# Version: 0.001
# Updates:
"""
Class to manipulate dates for display purposes
"""

__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "ezra.peisach@rcsb.org"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import datetime


class DateUtil(object):
    def __init__(self):
        pass

    @staticmethod
    def date_to_display(date):
        """Converts incoming dateIn from ISO 8601 YYYY-MM-DD to Month number, year
        If dateIn is '.' or '?' returns [UNKNOWN]. If date cannot be parsed, returns date.
        This should be picked up by dictionary checks.
        """

        ret = "[UNKNOWN]"
        if date not in [".", "?"]:
            try:
                dt = datetime.datetime.strptime(date, "%Y-%m-%d")
                ret = dt.strftime("%-d %B %Y")
            except ValueError:
                ret = date

        return ret

    @staticmethod
    def datetime_to_display(date):
        """Converts a datetime.date object to ISO 8601 format"""
        try:
            dt = date.strftime("%-d %B %Y")
        except ValueError:
            dt = date
        return dt


def main():
    du = DateUtil()
    print(du.date_to_display("2018-10-08"))
    print(du.date_to_display("2018-10"))
    print(du.date_to_display("."))
    dt = datetime.date(2018, 10, 3)
    print(du.datetime_to_display(dt))


if __name__ == "__main__":
    main()



##
# File:  AutoMessage.py
# Date:  28-Jun-2019 E. Peisach
#
# Update:
"""
Support for automatic scripts to send template drive email messages and archive in normal message stream/notes


"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "peisach@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"

import sys
import logging
from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId
from wwpdb.apps.msgmodule.io.MessagingIo import MessagingIo
from wwpdb.utils.session.WebRequest import InputRequest
from mmcif.io.IoAdapterCore import IoAdapterCore
from wwpdb.io.locator.PathInfo import PathInfo

#

logger = logging.getLogger(__name__)


class AutoMessage(object):
    def __init__(self, siteId=None, topSessionDir=None, verbose=False, log=sys.stderr):
        if siteId:
            self.__siteId = siteId
        else:
            self.__siteId = getSiteId()
        logger.debug("Site id is %s", self.__siteId)
        self.__cI = ConfigInfo(self.__siteId)
        self.__verbose = verbose
        self.__log = log

        if topSessionDir:
            self.__topSessionDir = topSessionDir
        else:
            self.__topSessionDir = self.__cI.get("SITE_WEB_APPS_TOP_SESSIONS_PATH")

    def __getmsgio(self):
        paramdict = {"TopSessionPath": [self.__topSessionDir]}
        reqobj = InputRequest(paramdict, verbose=True)
        reqobj.setValue("WWPDB_SITE_ID", self.__siteId)
        # Session dir

        mio = MessagingIo(reqobj, verbose=self.__verbose, log=self.__log)
        return mio

    def sendRemindUnlocked(self, depidlist):
        self._sendReminderBulk(depidlist, p_tmplt="remind-unlocked")

    def sendImplicitApproved(self, depidlist):
        self._sendReminderBulk(depidlist, p_tmplt="implicit-approved")

    def _sendReminderBulk(self, depidlist, p_tmplt):
        """Sends the bulk messages - handling setting EM flag"""

        (pdbents, emdents) = self._splitents(depidlist)

        mio = self.__getmsgio()
        if pdbents:
            mio.autoMsg(pdbents, p_tmpltType=p_tmplt)
        if emdents:
            mio.autoMsg(emdents, p_tmpltType=p_tmplt, p_isEmdbEntry=True)

    def _getExptl(self, depid):
        ctrgs = ["exptl"]

        ret = []
        try:
            pi = PathInfo(siteId=self.__siteId)
            modelpath = pi.getModelPdbxFilePath(dataSetId=depid, fileSource="archive")
            pr = IoAdapterCore()
            containerl = pr.readFile(inputFilePath=modelpath, selectList=ctrgs)
            if len(containerl) > 0:
                block0 = containerl[0]
                cobj = block0.getObj("exptl")
                if cobj and cobj.hasAttribute("method"):
                    for row in range(cobj.getRowCount()):
                        ret.append(cobj.getValue("method", row))
        except Exception as _e:  # noqa: F841
            logger.exception("Failed to parse model file")

        return ret

    def _splitents(self, depidlist):
        ements = []
        pdbents = []
        for depid in depidlist:
            expmeths = self._getExptl(depid)
            if "ELECTRON MICROSCOPY" in expmeths:
                ements.append(depid)
            else:
                pdbents.append(depid)
        return (pdbents, ements)
